/* ================================================================================ */
/*   driver/at24c.c                                                                 */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#include "rtos.h"
#include "i2c.h"


static __FAR uint8_t				m_data[2] ;						/* f[^   */
static __FAR I2CINFO				m_i2cInfo[2] ;					/* I2C  */


/* -------------------------------------------------------------------------------- */
/*   f[^ǂݍ                                                                 */
/*                                                                                  */
/*   IN : saddr  = X[uAhX                                                 */
/*        daddr  = f[^AhX                                                   */
/*        p_data = f[^i[AhX                                               */
/*        count  = f[^                                                         */
/*   OUT: */
/* -------------------------------------------------------------------------------- */
int8_t at24c_read(uint8_t saddr, uint16_t daddr, void __FAR *p_data, uint8_t count)
{
	int8_t	result ;


	AT24C_LOCK() ;

	m_data[0]           = (uint8_t )(daddr >> 8) ;
	m_data[1]           = (uint8_t )(daddr >> 0) ;
	m_i2cInfo[0].saddr  = saddr ;
	m_i2cInfo[0].flag   = I2CINFO_FLAG_WRITE | I2CINFO_FLAG_NOSTOP ;
	m_i2cInfo[0].count  = 2 ;
	m_i2cInfo[0].p_data = &m_data[0] ;
	m_i2cInfo[1].saddr  = saddr ;
	m_i2cInfo[1].flag   = I2CINFO_FLAG_WRITE | I2CINFO_FLAG_READ ;
	m_i2cInfo[1].count  = count ;
	m_i2cInfo[1].p_data = &p_data[0] ;
	result              = AT24C_XFER(&m_i2cInfo[0], 2) ;

	AT24C_UNLOCK() ;

	return result ;
}


/* -------------------------------------------------------------------------------- */
/*   JgAhX̃f[^ǂݍ                                               */
/*                                                                                  */
/*   IN : saddr  = X[uAhX                                                 */
/*        p_data = f[^i[AhX                                               */
/*        count  = f[^                                                         */
/*   OUT: */
/* -------------------------------------------------------------------------------- */
int8_t at24c_cread(uint8_t saddr, void __FAR *p_data, uint8_t count)
{
	int8_t	result ;


	AT24C_LOCK() ;

	result = AT24C_RECV(saddr, p_data, count) ;

	AT24C_UNLOCK() ;

	return result ;
}


/* -------------------------------------------------------------------------------- */
/*   f[^                                                                 */
/*                                                                                  */
/*   IN : saddr  = X[uAhX                                                 */
/*        daddr  = f[^AhX                                                   */
/*        p_data = f[^i[AhX                                               */
/*        count  = f[^                                                         */
/*   OUT: */
/* -------------------------------------------------------------------------------- */
int8_t at24c_write(uint8_t saddr, uint16_t daddr, const void __FAR *p_data, uint8_t count)
{
	int8_t		result ;
	uint8_t		index ;


	AT24C_LOCK() ;

	do {
		m_data[0]           = (uint8_t )(daddr >> 8) ;
		m_data[1]           = (uint8_t )(daddr >> 0) ;
		m_i2cInfo[0].saddr  = saddr ;
		m_i2cInfo[0].flag   = I2CINFO_FLAG_WRITE | I2CINFO_FLAG_NOSTOP ;
		m_i2cInfo[0].count  = 2 ;
		m_i2cInfo[0].p_data = &m_data[0] ;
		m_i2cInfo[1].saddr  = saddr ;
		m_i2cInfo[1].flag   = I2CINFO_FLAG_WRITE | I2CINFO_FLAG_NOADDR ;
		m_i2cInfo[1].count  = count ;
		m_i2cInfo[1].p_data = &p_data[0] ;
		result              = AT24C_XFER(&m_i2cInfo[0], 2) ;
		if( result || !count ) {
			break ;
		}

		/* ݊҂ */
		for(index=0; index<20; index++) {
			result = AT24C_SEND(saddr, NULL, 0) ;
			if( result != I2C_E_NACK ) {
				break ;
			}
			dly_tsk( 1 ) ;
		}
	} while( 0 ) ;

	AT24C_UNLOCK() ;

	return result ;
}
