/* ================================================================================ */
/*   driver/ili9325.h                                                               */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#ifndef __DRIVER__ILI9325_H__INCLUDED__
#define __DRIVER__ILI9325_H__INCLUDED__


	enum {
		ILI9325_R00						=	0x00,					/* Driver Code Read */
		ILI9325_R01						=	0x01,					/* Driver Output Control 1 */
			ILI9325_R01_SM				=	1 << 10,
			ILI9325_R01_SS				=	1 << 8,
		ILI9325_R02						=	0x02,					/* LCD Driving Control */
			ILI9325_R02_1				=	1 << 10,
			ILI9325_R02_BC				=	1 << 9,
			ILI9325_R02_EOR				=	1 << 8,
		ILI9325_R03						=	0x03,					/* Entry Mode */
			ILI9325_R03_TRI				=	1 << 15,
			ILI9325_R03_DFM				=	1 << 14,
			ILI9325_R03_BGR				=	1 << 12,
			ILI9325_R03_HWM				=	1 << 9,
			ILI9325_R03_ORG				=	1 << 7,
			ILI9325_R03_AM				=	1 << 3,
		ILI9325_R04						=	0x04,					/* Resize Control */
			ILI9325_R04_RCV_0PIXEL		=	0 << 8,
			ILI9325_R04_RCV_1PIXEL		=	1 << 8,
			ILI9325_R04_RCV_2PIXEL		=	2 << 8,
			ILI9325_R04_RCV_3PIXEL		=	3 << 8,
			ILI9325_R04_RCH_0PIXEL		=	0 << 4,
			ILI9325_R04_RCH_1PIXEL		=	1 << 4,
			ILI9325_R04_RCH_2PIXEL		=	2 << 4,
			ILI9325_R04_RCH_3PIXEL		=	3 << 4,
			ILI9325_R04_RSZ_x1			=	0 << 0,
			ILI9325_R04_RSZ_x1_2		=	1 << 0,
			ILI9325_R04_RSZ_x1_4		=	3 << 0,
		ILI9325_R07						=	0x07,					/* Display Control 1 */
			ILI9325_R07_BASEE			=	1 << 8,
			ILI9325_R07_GON				=	1 << 5,
			ILI9325_R07_DTE				=	1 << 4,
			ILI9325_R07_CL_262K			=	0 << 3,
			ILI9325_R07_CL_8			=	1 << 3,
		ILI9325_R08						=	0x08,					/* Display Control 2 */
			ILI9325_R08_FP_2LINES		=	 2 << 8,
			ILI9325_R08_FP_3LINES		=	 3 << 8,
			ILI9325_R08_FP_4LINES		=	 4 << 8,
			ILI9325_R08_FP_5LINES		=	 5 << 8,
			ILI9325_R08_FP_6LINES		=	 6 << 8,
			ILI9325_R08_FP_7LINES		=	 7 << 8,
			ILI9325_R08_FP_8LINES		=	 8 << 8,
			ILI9325_R08_FP_9LINES		=	 9 << 8,
			ILI9325_R08_FP_10LINES		=	10 << 8,
			ILI9325_R08_FP_11LINES		=	11 << 8,
			ILI9325_R08_FP_12LINES		=	12 << 8,
			ILI9325_R08_FP_13LINES		=	13 << 8,
			ILI9325_R08_FP_14LINES		=	14 << 8,
			ILI9325_R08_BP_2LINES		=	 2 << 0,
			ILI9325_R08_BP_3LINES		=	 3 << 0,
			ILI9325_R08_BP_4LINES		=	 4 << 0,
			ILI9325_R08_BP_5LINES		=	 5 << 0,
			ILI9325_R08_BP_6LINES		=	 6 << 0,
			ILI9325_R08_BP_7LINES		=	 7 << 0,
			ILI9325_R08_BP_8LINES		=	 8 << 0,
			ILI9325_R08_BP_9LINES		=	 9 << 0,
			ILI9325_R08_BP_10LINES		=	10 << 0,
			ILI9325_R08_BP_11LINES		=	11 << 0,
			ILI9325_R08_BP_12LINES		=	12 << 0,
			ILI9325_R08_BP_13LINES		=	13 << 0,
			ILI9325_R08_BP_14LINES		=	14 << 0,
		ILI9325_R09						=	0x09,					/* Display Control 3 */
			ILI9325_R09_PTS_RS_V63_V0	=	 0 << 8,
			ILI9325_R09_PTS_RS_GND_GND	=	 2 << 8,
			ILI9325_R09_PTS_RS_HIZ_HIZ	=	 3 << 8,
			ILI9325_R09_PTS_FS_V63_V0	=	 4 << 8,
			ILI9325_R09_PTS_FS_GND_GND	=	 6 << 8,
			ILI9325_R09_PTS_FS_HIZ_HIZ	=	 7 << 8,
			ILI9325_R09_PTG_NORMAL		=	 0 << 4,
			ILI9325_R09_PTG_INTERVAL	=	 2 << 4,
			ILI9325_R09_ISC_0FRAME		=	 0 << 0,
			ILI9325_R09_ISC_1FRAME		=	 1 << 0,
			ILI9325_R09_ISC_3FRAME		=	 2 << 0,
			ILI9325_R09_ISC_5FRAME		=	 3 << 0,
			ILI9325_R09_ISC_7FRAME		=	 4 << 0,
			ILI9325_R09_ISC_9FRAME		=	 5 << 0,
			ILI9325_R09_ISC_11FRAME		=	 6 << 0,
			ILI9325_R09_ISC_13FRAME		=	 7 << 0,
			ILI9325_R09_ISC_15FRAME		=	 8 << 0,
			ILI9325_R09_ISC_17FRAME		=	 9 << 0,
			ILI9325_R09_ISC_19FRAME		=	10 << 0,
			ILI9325_R09_ISC_21FRAME		=	11 << 0,
			ILI9325_R09_ISC_23FRAME		=	12 << 0,
			ILI9325_R09_ISC_25FRAME		=	13 << 0,
			ILI9325_R09_ISC_27FRAME		=	14 << 0,
			ILI9325_R09_ISC_29FRAME		=	15 << 0,
		ILI9325_R0A						=	0x0a,					/* Display Control 4 */
			ILI9325_R0A_FMARKOE			=	1 << 3,
			ILI9325_R0A_FMI_1FRAME		=	0 << 0,
			ILI9325_R0A_FMI_2FRAME		=	1 << 0,
			ILI9325_R0A_FMI_4FRAME		=	3 << 0,
			ILI9325_R0A_FMI_6FRAME		=	5 << 0,
		ILI9325_R0C						=	0x0c,					/* RGB Display Interface Control 1 */
			ILI9325_R0C_ENC_1FRAMES		=	0 << 12,
			ILI9325_R0C_ENC_2FRAMES		=	1 << 12,
			ILI9325_R0C_ENC_3FRAMES		=	2 << 12,
			ILI9325_R0C_ENC_4FRAMES		=	3 << 12,
			ILI9325_R0C_ENC_5FRAMES		=	4 << 12,
			ILI9325_R0C_ENC_6FRAMES		=	5 << 12,
			ILI9325_R0C_ENC_7FRAMES		=	6 << 12,
			ILI9325_R0C_ENC_8FRAMES		=	7 << 12,
			ILI9325_R0C_RM				=	1 << 8,
			ILI9325_R0C_DM_ISYSCLK		=	0 << 4,
			ILI9325_R0C_DM_RGB			=	1 << 4,
			ILI9325_R0C_DM_VSYNC		=	2 << 4,
			ILI9325_R0C_RIM_RGB18BIT	=	0 << 0,
			ILI9325_R0C_RIM_RGB16BIT	=	1 << 0,
			ILI9325_R0C_RIM_RGB6BIT		=	2 << 0,
		ILI9325_R0D						=	0x0d,					/* Frame Maker Position */
		ILI9325_R0F						=	0x0f,					/* RGB Display Interface Control 2 */
			ILI9325_R0F_VSPL_LOW		=	0 << 4,
			ILI9325_R0F_VSPL_HIGH		=	1 << 4,
			ILI9325_R0F_HSPL_LOW		=	0 << 3,
			ILI9325_R0F_HSPL_HIGH		=	1 << 3,
			ILI9325_R0F_EPL				=	1 << 1,
			ILI9325_R0F_DPL_RISINGEDGE	=	0 << 0,
			ILI9325_R0F_DPL_FALLINGEDGE	=	1 << 0,
		ILI9325_R10						=	0x10,					/* Power Control 1 */
			ILI9325_R10_SAP				=	1 << 12,
			ILI9325_R10_APE				=	1 << 7,
			ILI9325_R10_DSTB			=	1 << 2,
			ILI9325_R10_SLP				=	1 << 1,
			ILI9325_R10_STB				=	1 << 0,
		ILI9325_R11						=	0x11,					/* Power Control 2 */
			ILI9325_R11_DC1_FOSC_4		=	0 << 8,
			ILI9325_R11_DC1_FOSC_8		=	1 << 8,
			ILI9325_R11_DC1_FOSC_16		=	2 << 8,
			ILI9325_R11_DC1_FOSC_32		=	3 << 8,
			ILI9325_R11_DC1_FOSC_64		=	4 << 8,
			ILI9325_R11_DC1_FOSC_128	=	5 << 8,
			ILI9325_R11_DC1_FOSC_256	=	6 << 8,
			ILI9325_R11_DC1_FOSC_HSC2	=	7 << 8,
			ILI9325_R11_DC0_FOSC		=	0 << 4,
			ILI9325_R11_DC0_FOSC_2		=	1 << 4,
			ILI9325_R11_DC0_FOSC_4		=	2 << 4,
			ILI9325_R11_DC0_FOSC_8		=	3 << 4,
			ILI9325_R11_DC0_FOSC_16		=	4 << 4,
			ILI9325_R11_DC0_FOSC_32		=	5 << 4,
			ILI9325_R11_DC0_FOSC_64		=	6 << 4,
			ILI9325_R11_DC0_FOSC_HSC1	=	7 << 4,
			ILI9325_R11_VC_0_95			=	0 << 0,
			ILI9325_R11_VC_0_90			=	1 << 0,
			ILI9325_R11_VC_0_85			=	2 << 0,
			ILI9325_R11_VC_0_80			=	3 << 0,
			ILI9325_R11_VC_0_75			=	4 << 0,
			ILI9325_R11_VC_0_70			=	5 << 0,
			ILI9325_R11_VC_1_00			=	7 << 0,
		ILI9325_R12						=	0x12,					/* Power Control 3 */
			ILI9325_R12_VCIRE			=	1 << 7,
			ILI9325_R12_PON_DISABLE		=	0 << 4,
			ILI9325_R12_PON_ENABLE		=	1 << 4,
		ILI9325_R13						=	0x13,					/* Power Control 4 */
			ILI9325_R13_VDV_x0_70		=	 0 << 8,
			ILI9325_R13_VDV_x0_72		=	 1 << 8,
			ILI9325_R13_VDV_x0_74		=	 2 << 8,
			ILI9325_R13_VDV_x0_76		=	 3 << 8,
			ILI9325_R13_VDV_x0_78		=	 4 << 8,
			ILI9325_R13_VDV_x0_80		=	 5 << 8,
			ILI9325_R13_VDV_x0_82		=	 6 << 8,
			ILI9325_R13_VDV_x0_84		=	 7 << 8,
			ILI9325_R13_VDV_x0_86		=	 8 << 8,
			ILI9325_R13_VDV_x0_88		=	 9 << 8,
			ILI9325_R13_VDV_x0_90		=	10 << 8,
			ILI9325_R13_VDV_x0_92		=	11 << 8,
			ILI9325_R13_VDV_x0_94		=	12 << 8,
			ILI9325_R13_VDV_x0_96		=	13 << 8,
			ILI9325_R13_VDV_x0_98		=	14 << 8,
			ILI9325_R13_VDV_x1_00		=	15 << 8,
/*			ILI9325_R13_VDV_x0_94		=	16 << 8,	*/
/*			ILI9325_R13_VDV_x0_96		=	17 << 8,	*/
/*			ILI9325_R13_VDV_x0_98		=	18 << 8,	*/
/*			ILI9325_R13_VDV_x1_00		=	19 << 8,	*/
			ILI9325_R13_VDV_x1_02		=	20 << 8,
			ILI9325_R13_VDV_x1_04		=	21 << 8,
			ILI9325_R13_VDV_x1_06		=	22 << 8,
			ILI9325_R13_VDV_x1_08		=	23 << 8,
			ILI9325_R13_VDV_x1_10		=	24 << 8,
			ILI9325_R13_VDV_x1_12		=	25 << 8,
			ILI9325_R13_VDV_x1_14		=	26 << 8,
			ILI9325_R13_VDV_x1_16		=	27 << 8,
			ILI9325_R13_VDV_x1_18		=	28 << 8,
			ILI9325_R13_VDV_x1_20		=	29 << 8,
			ILI9325_R13_VDV_x1_22		=	30 << 8,
			ILI9325_R13_VDV_x1_24		=	31 << 8,
		ILI9325_R20						=	0x20,					/* Horizontal GRAM Address Set */
		ILI9325_R21						=	0x21,					/* Vertical GRAM Address Set */
		ILI9325_R22						=	0x22,					/* Write Data to GRAM */
		ILI9325_R29						=	0x29,					/* Power Control 7 */
			ILI9325_R29_VCM_x0_685		=	 0 << 0,
			ILI9325_R29_VCM_x0_690		=	 1 << 0,
			ILI9325_R29_VCM_x0_695		=	 2 << 0,
			ILI9325_R29_VCM_x0_700		=	 3 << 0,
			ILI9325_R29_VCM_x0_705		=	 4 << 0,
			ILI9325_R29_VCM_x0_710		=	 5 << 0,
			ILI9325_R29_VCM_x0_715		=	 6 << 0,
			ILI9325_R29_VCM_x0_720		=	 7 << 0,
			ILI9325_R29_VCM_x0_725		=	 8 << 0,
			ILI9325_R29_VCM_x0_730		=	 9 << 0,
			ILI9325_R29_VCM_x0_735		=	10 << 0,
			ILI9325_R29_VCM_x0_740		=	11 << 0,
			ILI9325_R29_VCM_x0_745		=	12 << 0,
			ILI9325_R29_VCM_x0_750		=	13 << 0,
			ILI9325_R29_VCM_x0_755		=	14 << 0,
			ILI9325_R29_VCM_x0_760		=	15 << 0,
			ILI9325_R29_VCM_x0_765		=	16 << 0,
			ILI9325_R29_VCM_x0_770		=	17 << 0,
			ILI9325_R29_VCM_x0_775		=	18 << 0,
			ILI9325_R29_VCM_x0_780		=	19 << 0,
			ILI9325_R29_VCM_x0_785		=	20 << 0,
			ILI9325_R29_VCM_x0_790		=	21 << 0,
			ILI9325_R29_VCM_x0_795		=	22 << 0,
			ILI9325_R29_VCM_x0_800		=	23 << 0,
			ILI9325_R29_VCM_x0_805		=	24 << 0,
			ILI9325_R29_VCM_x0_810		=	25 << 0,
			ILI9325_R29_VCM_x0_815		=	26 << 0,
			ILI9325_R29_VCM_x0_820		=	27 << 0,
			ILI9325_R29_VCM_x0_825		=	28 << 0,
			ILI9325_R29_VCM_x0_830		=	29 << 0,
			ILI9325_R29_VCM_x0_835		=	30 << 0,
			ILI9325_R29_VCM_x0_840		=	31 << 0,
			ILI9325_R29_VCM_x0_845		=	32 << 0,
			ILI9325_R29_VCM_x0_850		=	33 << 0,
			ILI9325_R29_VCM_x0_855		=	34 << 0,
			ILI9325_R29_VCM_x0_860		=	35 << 0,
			ILI9325_R29_VCM_x0_865		=	36 << 0,
			ILI9325_R29_VCM_x0_870		=	37 << 0,
			ILI9325_R29_VCM_x0_875		=	38 << 0,
			ILI9325_R29_VCM_x0_880		=	39 << 0,
			ILI9325_R29_VCM_x0_885		=	40 << 0,
			ILI9325_R29_VCM_x0_890		=	41 << 0,
			ILI9325_R29_VCM_x0_895		=	42 << 0,
			ILI9325_R29_VCM_x0_900		=	43 << 0,
			ILI9325_R29_VCM_x0_905		=	44 << 0,
			ILI9325_R29_VCM_x0_910		=	45 << 0,
			ILI9325_R29_VCM_x0_915		=	46 << 0,
			ILI9325_R29_VCM_x0_920		=	47 << 0,
			ILI9325_R29_VCM_x0_925		=	48 << 0,
			ILI9325_R29_VCM_x0_930		=	49 << 0,
			ILI9325_R29_VCM_x0_935		=	50 << 0,
			ILI9325_R29_VCM_x0_940		=	51 << 0,
			ILI9325_R29_VCM_x0_945		=	52 << 0,
			ILI9325_R29_VCM_x0_950		=	53 << 0,
			ILI9325_R29_VCM_x0_955		=	54 << 0,
			ILI9325_R29_VCM_x0_960		=	55 << 0,
			ILI9325_R29_VCM_x0_965		=	56 << 0,
			ILI9325_R29_VCM_x0_970		=	57 << 0,
			ILI9325_R29_VCM_x0_975		=	58 << 0,
			ILI9325_R29_VCM_x0_980		=	59 << 0,
			ILI9325_R29_VCM_x0_985		=	60 << 0,
			ILI9325_R29_VCM_x0_990		=	61 << 0,
			ILI9325_R29_VCM_x0_995		=	62 << 0,
			ILI9325_R29_VCM_x1_000		=	63 << 0,
		ILI9325_R2B						=	0x2b,					/* Frame Rate and Color Control */
			ILI9325_R2B_FRS_30			=	 0 << 0,
			ILI9325_R2B_FRS_31			=	 1 << 0,
			ILI9325_R2B_FRS_33			=	 2 << 0,
			ILI9325_R2B_FRS_35			=	 3 << 0,
			ILI9325_R2B_FRS_38			=	 4 << 0,
			ILI9325_R2B_FRS_40			=	 5 << 0,
			ILI9325_R2B_FRS_43			=	 6 << 0,
			ILI9325_R2B_FRS_47			=	 7 << 0,
			ILI9325_R2B_FRS_51			=	 8 << 0,
			ILI9325_R2B_FRS_56			=	 9 << 0,
			ILI9325_R2B_FRS_62			=	10 << 0,
			ILI9325_R2B_FRS_70			=	11 << 0,
			ILI9325_R2B_FRS_80			=	12 << 0,
			ILI9325_R2B_FRS_93			=	13 << 0,
			ILI9325_R2B_FRS_112			=	14 << 0,
		ILI9325_R30						=	0x30,					/* Gamma Control 1 */
		ILI9325_R31						=	0x31,					/* Gamma Control 2 */
		ILI9325_R32						=	0x32,					/* Gamma Control 3 */
		ILI9325_R35						=	0x35,					/* Gamma Control 4 */
		ILI9325_R36						=	0x36,					/* Gamma Control 5 */
		ILI9325_R37						=	0x37,					/* Gamma Control 6 */
		ILI9325_R38						=	0x38,					/* Gamma Control 7 */
		ILI9325_R39						=	0x39,					/* Gamma Control 8 */
		ILI9325_R3C						=	0x3c,					/* Gamma Control 9 */
		ILI9325_R3D						=	0x3d,					/* Gamma Control 10 */
		ILI9325_R50						=	0x50,					/* Horizontal Address Start */
		ILI9325_R51						=	0x51,					/* Horizontal Address End Position */
		ILI9325_R52						=	0x52,					/* Vertical Address Start Position */
		ILI9325_R53						=	0x53,					/* Vertical Address End Position */
		ILI9325_R60						=	0x60,					/* Driver Output Control 2 */
			ILI9325_R60_GS				=	1 << 15,
			ILI9325_R60_NL_240			=	0x1d << 8,
			ILI9325_R60_NL_248			=	0x1e << 8,
			ILI9325_R60_NL_256			=	0x1f << 8,
			ILI9325_R60_NL_264			=	0x20 << 8,
			ILI9325_R60_NL_272			=	0x21 << 8,
			ILI9325_R60_NL_280			=	0x22 << 8,
			ILI9325_R60_NL_288			=	0x23 << 8,
			ILI9325_R60_NL_296			=	0x24 << 8,
			ILI9325_R60_NL_304			=	0x25 << 8,
			ILI9325_R60_NL_312			=	0x26 << 8,
			ILI9325_R60_NL_320			=	0x27 << 8,
		ILI9325_R61						=	0x61,					/* Base Image Display Control */
			ILI9325_R61_NDL				=	1 << 2,
			ILI9325_R61_VLE				=	1 << 1,
			ILI9325_R61_REV				=	1 << 0,
		ILI9325_R6A						=	0x6a,					/* Vertical Scroll Control */
		ILI9325_R80						=	0x80,					/* Partial Image 1 Display Position */
		ILI9325_R81						=	0x81,					/* Partial Image 1 Area (Start Line) */
		ILI9325_R82						=	0x82,					/* Partial Image 1 Area (End Line) */
		ILI9325_R83						=	0x83,					/* Partial Image 2 Display Position */
		ILI9325_R84						=	0x84,					/* Partial Image 2 Area (Start Line) */
		ILI9325_R85						=	0x85,					/* Partial Image 2 Area (End Line) */
		ILI9325_R90						=	0x90,					/* Panel Interface Control 1 */
			ILI9325_R90_DIVI_FOSC_1		=	0 << 8,
			ILI9325_R90_DIVI_FOSC_2		=	1 << 8,
			ILI9325_R90_DIVI_FOSC_4		=	2 << 8,
			ILI9325_R90_DIVI_FOSC_8		=	3 << 8,
			ILI9325_R90_RTNI_16CLK		=	16 << 0,
			ILI9325_R90_RTNI_17CLK		=	17 << 0,
			ILI9325_R90_RTNI_18CLK		=	18 << 0,
			ILI9325_R90_RTNI_19CLK		=	19 << 0,
			ILI9325_R90_RTNI_20CLK		=	20 << 0,
			ILI9325_R90_RTNI_21CLK		=	21 << 0,
			ILI9325_R90_RTNI_22CLK		=	22 << 0,
			ILI9325_R90_RTNI_23CLK		=	23 << 0,
			ILI9325_R90_RTNI_24CLK		=	24 << 0,
			ILI9325_R90_RTNI_25CLK		=	25 << 0,
			ILI9325_R90_RTNI_26CLK		=	26 << 0,
			ILI9325_R90_RTNI_27CLK		=	27 << 0,
			ILI9325_R90_RTNI_28CLK		=	28 << 0,
			ILI9325_R90_RTNI_29CLK		=	29 << 0,
			ILI9325_R90_RTNI_30CLK		=	30 << 0,
			ILI9325_R90_RTNI_31CLK		=	31 << 0,
		ILI9325_R92						=	0x92,					/* Panel Interface Control 2 */
			ILI9325_R92_NOWI_0CLK		=	0 << 8,
			ILI9325_R92_NOWI_1CLK		=	1 << 8,
			ILI9325_R92_NOWI_2CLK		=	2 << 8,
			ILI9325_R92_NOWI_3CLK		=	3 << 8,
			ILI9325_R92_NOWI_4CLK		=	4 << 8,
			ILI9325_R92_NOWI_5CLK		=	5 << 8,
			ILI9325_R92_NOWI_6CLK		=	6 << 8,
			ILI9325_R92_NOWI_7CLK		=	7 << 8,
		ILI9325_R95						=	0x95,					/* Panel Interface Control 4 */
			ILI9325_R95_RTNE_16CLK		=	0x10 << 0,
			ILI9325_R95_RTNE_17CLK		=	0x11 << 0,
			ILI9325_R95_RTNE_18CLK		=	0x12 << 0,
			ILI9325_R95_RTNE_19CLK		=	0x13 << 0,
			ILI9325_R95_RTNE_20CLK		=	0x14 << 0,
			ILI9325_R95_RTNE_21CLK		=	0x15 << 0,
			ILI9325_R95_RTNE_22CLK		=	0x16 << 0,
			ILI9325_R95_RTNE_23CLK		=	0x17 << 0,
			ILI9325_R95_RTNE_24CLK		=	0x18 << 0,
			ILI9325_R95_RTNE_25CLK		=	0x19 << 0,
			ILI9325_R95_RTNE_26CLK		=	0x1a << 0,
			ILI9325_R95_RTNE_27CLK		=	0x1b << 0,
			ILI9325_R95_RTNE_28CLK		=	0x1c << 0,
			ILI9325_R95_RTNE_29CLK		=	0x1d << 0,
			ILI9325_R95_RTNE_30CLK		=	0x1e << 0,
			ILI9325_R95_RTNE_31CLK		=	0x1f << 0,
			ILI9325_R95_RTNE_32CLK		=	0x20 << 0,
			ILI9325_R95_RTNE_33CLK		=	0x21 << 0,
			ILI9325_R95_RTNE_34CLK		=	0x22 << 0,
			ILI9325_R95_RTNE_35CLK		=	0x23 << 0,
			ILI9325_R95_RTNE_36CLK		=	0x24 << 0,
			ILI9325_R95_RTNE_37CLK		=	0x25 << 0,
			ILI9325_R95_RTNE_38CLK		=	0x26 << 0,
			ILI9325_R95_RTNE_39CLK		=	0x27 << 0,
			ILI9325_R95_RTNE_40CLK		=	0x28 << 0,
			ILI9325_R95_RTNE_41CLK		=	0x29 << 0,
			ILI9325_R95_RTNE_42CLK		=	0x2a << 0,
			ILI9325_R95_RTNE_43CLK		=	0x2b << 0,
			ILI9325_R95_RTNE_44CLK		=	0x2c << 0,
			ILI9325_R95_RTNE_45CLK		=	0x2d << 0,
			ILI9325_R95_RTNE_46CLK		=	0x2e << 0,
			ILI9325_R95_RTNE_47CLK		=	0x2f << 0,
			ILI9325_R95_RTNE_48CLK		=	0x30 << 0,
			ILI9325_R95_RTNE_49CLK		=	0x31 << 0,
			ILI9325_R95_RTNE_50CLK		=	0x32 << 0,
			ILI9325_R95_RTNE_51CLK		=	0x33 << 0,
			ILI9325_R95_RTNE_52CLK		=	0x34 << 0,
			ILI9325_R95_RTNE_53CLK		=	0x35 << 0,
			ILI9325_R95_RTNE_54CLK		=	0x36 << 0,
			ILI9325_R95_RTNE_55CLK		=	0x37 << 0,
			ILI9325_R95_RTNE_56CLK		=	0x38 << 0,
			ILI9325_R95_RTNE_57CLK		=	0x39 << 0,
			ILI9325_R95_RTNE_58CLK		=	0x3a << 0,
			ILI9325_R95_RTNE_59CLK		=	0x3b << 0,
			ILI9325_R95_RTNE_60CLK		=	0x3c << 0,
			ILI9325_R95_RTNE_61CLK		=	0x3d << 0,
			ILI9325_R95_RTNE_62CLK		=	0x3e << 0,
			ILI9325_R95_RTNE_63CLK		=	0x3f << 0,
		ILI9325_R97						=	0x97,					/* Panel Interface Control 5 */
			ILI9325_R97_NOWE_0CLK		=	 0 << 8,
			ILI9325_R97_NOWE_1CLK		=	 1 << 8,
			ILI9325_R97_NOWE_2CLK		=	 2 << 8,
			ILI9325_R97_NOWE_3CLK		=	 3 << 8,
			ILI9325_R97_NOWE_4CLK		=	 4 << 8,
			ILI9325_R97_NOWE_5CLK		=	 5 << 8,
			ILI9325_R97_NOWE_6CLK		=	 6 << 8,
			ILI9325_R97_NOWE_7CLK		=	 7 << 8,
			ILI9325_R97_NOWE_8CLK		=	 8 << 8,
			ILI9325_R97_NOWE_9CLK		=	 9 << 8,
			ILI9325_R97_NOWE_10CLK		=	10 << 8,
			ILI9325_R97_NOWE_11CLK		=	11 << 8,
			ILI9325_R97_NOWE_12CLK		=	12 << 8,
			ILI9325_R97_NOWE_13CLK		=	13 << 8,
			ILI9325_R97_NOWE_14CLK		=	14 << 8,
			ILI9325_R97_NOWE_15CLK		=	15 << 8,
		ILI9325_RA1						=	0xa1,					/* OTP VCM Programming Control */
			ILI9325_RA1_OTP_PGM_EN		=	1 << 11,
		ILI9325_RA2						=	0xa2,					/* OTP VCM Status and Enable */
			ILI9325_RA2_PGM_CNT_OTPC	=	0 << 14,
			ILI9325_RA2_PGM_CNT_OTP1	=	1 << 14,
			ILI9325_RA2_PGM_CNT_OTP2	=	2 << 14,
			ILI9325_RA2_PGM_CNT_OTP3	=	3 << 14,
			ILI9325_RA2_VCM_EN			=	1 << 0,
		ILI9325_RA5						=	0xa5,					/* OTP Programming ID Key */

		ILI9325_WMS						=	ILI9325_R01				/* ҂ԎwiPʂ ms ŏpj */
	} ;


#define ILI9325_R03_I_D( n )		((uint16_t )(n) <<  4)
#define ILI9325_R07_PTDE( n )		((uint16_t )(n) << 12)
#define ILI9325_R07_D( n )			((uint16_t )(n) <<  0)
#define ILI9325_R0D_FMP( n )		((uint16_t )(n) <<  0)
#define ILI9325_R10_BT( n )			((uint16_t )(n) <<  8)
#define ILI9325_R10_AP( n )			((uint16_t )(n) <<  4)
#define ILI9325_R12_VRH( n )		((uint16_t )(n) <<  0)
#define ILI9325_R30_KP1( n )		((uint16_t )(n) <<  8)
#define ILI9325_R30_KP0( n )		((uint16_t )(n) <<  0)
#define ILI9325_R31_KP3( n )		((uint16_t )(n) <<  8)
#define ILI9325_R31_KP2( n )		((uint16_t )(n) <<  0)
#define ILI9325_R32_KP5( n )		((uint16_t )(n) <<  8)
#define ILI9325_R32_KP4( n )		((uint16_t )(n) <<  0)
#define ILI9325_R35_RP1( n )		((uint16_t )(n) <<  8)
#define ILI9325_R35_RP0( n )		((uint16_t )(n) <<  0)
#define ILI9325_R36_VRP1( n )		((uint16_t )(n) <<  8)
#define ILI9325_R36_VRP0( n )		((uint16_t )(n) <<  0)
#define ILI9325_R37_KN1( n )		((uint16_t )(n) <<  8)
#define ILI9325_R37_KN0( n )		((uint16_t )(n) <<  0)
#define ILI9325_R38_KN3( n )		((uint16_t )(n) <<  8)
#define ILI9325_R38_KN2( n )		((uint16_t )(n) <<  0)
#define ILI9325_R39_KN5( n )		((uint16_t )(n) <<  8)
#define ILI9325_R39_KN4( n )		((uint16_t )(n) <<  0)
#define ILI9325_R3C_RN1( n )		((uint16_t )(n) <<  8)
#define ILI9325_R3C_RN0( n )		((uint16_t )(n) <<  0)
#define ILI9325_R3D_VRN1( n )		((uint16_t )(n) <<  8)
#define ILI9325_R3D_VRN0( n )		((uint16_t )(n) <<  0)
#define ILI9325_R60_SCN( n )		((uint16_t )(n) <<  0)
#define ILI9325_R80_PTDP0( n )		((uint16_t )(n) <<  0)
#define ILI9325_R81_PTSA0( n )		((uint16_t )(n) <<  0)
#define ILI9325_R82_PTEA0( n )		((uint16_t )(n) <<  0)
#define ILI9325_R83_PTDP1( n )		((uint16_t )(n) <<  0)
#define ILI9325_R84_PTSA1( n )		((uint16_t )(n) <<  0)
#define ILI9325_R85_PTEA1( n )		((uint16_t )(n) <<  0)
#define ILI9325_R95_DIVE( n )		((uint16_t )(n) <<  8)
#define ILI9325_RA1_VCM_OTP( n )	((uint16_t )(n) <<  0)
#define ILI9325_RA2_VCM_D( n )		((uint16_t )(n) <<  8)
#define ILI9325_RA5_KEY( n )		((uint16_t )(n) <<  0)

	/* WX^ */
	typedef struct {
		uint8_t						index ;
		uint16_t					value ;
	} ILI9325REG ;


extern void ili9325_init( const ILI9325REG __CODE *p_reg ) ;
extern void ili9325_writeIndex( uint8_t index ) ;

#ifdef  ILI9325_8BIT
extern void ili9325_writeData( uint16_t data ) ;
extern void ili9325_writeReg(uint8_t index, uint16_t data) ;
#endif/*ILI9325_8BIT*/


#endif/*__DRIVER__ILI9325_H__INCLUDED__*/
