/* ================================================================================ */
/*   driver/lm024c9325.c                                                            */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#include "rtos.h"
#include "lm024c9325.h"


const ILI9325REG					g_lm024c9325Reg[]={
										{ 0xe3,        0x3008                                                                                                     },	/* Set internal timing */
										{ 0xe7,        0x0012                                                                                                     },	/* Set internal timing */
										{ 0xef,        0x1231                                                                                                     },	/* Set internal timing */
										{ ILI9325_R01, ILI9325_R01_SS                                                                                             },
										{ ILI9325_R02, ILI9325_R02_1|ILI9325_R02_BC|ILI9325_R02_EOR                                                               },
										{ ILI9325_R03, ILI9325_R03_BGR|ILI9325_R03_I_D(3)                                                                         },
										{ ILI9325_R04, ILI9325_R04_RCV_0PIXEL|ILI9325_R04_RCH_0PIXEL|ILI9325_R04_RSZ_x1                                           },
										{ ILI9325_R08, ILI9325_R08_FP_2LINES|ILI9325_R08_BP_7LINES                                                                },
										{ ILI9325_R09, ILI9325_R09_PTS_RS_V63_V0|ILI9325_R09_PTG_NORMAL|ILI9325_R09_ISC_0FRAME                                    },
										{ ILI9325_R0A, 0                                                                                                          },
										{ ILI9325_R0C, ILI9325_R0C_ENC_1FRAMES|ILI9325_R0C_DM_ISYSCLK |ILI9325_R0C_RIM_RGB18BIT                                   },
										{ ILI9325_R0D, 0                                                                                                          },
										{ ILI9325_R0F, ILI9325_R0F_VSPL_LOW|ILI9325_R0F_HSPL_LOW|ILI9325_R0F_DPL_RISINGEDGE                                       },
										{ ILI9325_R10, ILI9325_R10_BT(0)|ILI9325_R10_AP(0)                                                                        },
										{ ILI9325_R11, ILI9325_R11_DC1_FOSC_4|ILI9325_R11_DC0_FOSC|ILI9325_R11_VC_1_00                                            },
										{ ILI9325_R12, ILI9325_R12_PON_DISABLE|ILI9325_R12_VRH(0)                                                                 },
										{ ILI9325_R13, ILI9325_R13_VDV_x0_70                                                                                      },
										{ ILI9325_WMS, 200                                                                                                        },
										{ ILI9325_R10, ILI9325_R10_SAP|ILI9325_R10_BT(4)|ILI9325_R10_APE|ILI9325_R10_AP(1)                                        },
										{ ILI9325_R11, ILI9325_R11_DC1_FOSC_16|ILI9325_R11_DC0_FOSC_4|ILI9325_R11_VC_1_00                                         },
										{ ILI9325_WMS, 50                                                                                                         },
										{ ILI9325_R12, ILI9325_R12_PON_ENABLE|ILI9325_R12_VRH(10)                                                                 },
										{ ILI9325_WMS, 50                                                                                                         },
										{ ILI9325_R13, ILI9325_R13_VDV_x1_02                                                                                      },
										{ ILI9325_R29, ILI9325_R29_VCM_x0_810                                                                                     },
										{ ILI9325_R2B, ILI9325_R2B_FRS_62                                                                                         },
										{ ILI9325_WMS, 50                                                                                                         },
										{ ILI9325_R20, 0                                                                                                          },
										{ ILI9325_R21, 0                                                                                                          },
										{ ILI9325_R30, ILI9325_R30_KP1(0)|ILI9325_R30_KP0(0)                                                                      },
										{ ILI9325_R31, ILI9325_R31_KP3(5)|ILI9325_R31_KP2(6)                                                                      },
										{ ILI9325_R32, ILI9325_R32_KP5(1)|ILI9325_R32_KP4(4)                                                                      },
										{ ILI9325_R35, ILI9325_R35_RP1(2)|ILI9325_R35_RP0(7)                                                                      },
										{ ILI9325_R36, ILI9325_R36_VRP1(0)|ILI9325_R36_VRP0(15)                                                                   },
										{ ILI9325_R37, ILI9325_R37_KN1(3)|ILI9325_R37_KN0(6)                                                                      },
										{ ILI9325_R38, ILI9325_R38_KN3(1)|ILI9325_R38_KN2(2)                                                                      },
										{ ILI9325_R39, ILI9325_R39_KN5(7)|ILI9325_R39_KN4(7)                                                                      },
										{ ILI9325_R3C, ILI9325_R3C_RN1(7)|ILI9325_R3C_RN0(2)                                                                      },
										{ ILI9325_R3D, ILI9325_R3D_VRN1(22)|ILI9325_R3D_VRN0(4)                                                                   },
										{ ILI9325_R50, 0                                                                                                          },
										{ ILI9325_R51, LM024C9325_SCREEN_WIDTH-1                                                                                  },
										{ ILI9325_R52, 0                                                                                                          },
										{ ILI9325_R53, LM024C9325_SCREEN_HEIGHT-1                                                                                 },
										{ ILI9325_R60, ILI9325_R60_NL_320                                                                                         },
										{ ILI9325_R61, ILI9325_R61_VLE|ILI9325_R61_REV                                                                            },
										{ ILI9325_R6A, 0                                                                                                          },
										{ ILI9325_R80, 0                                                                                                          },
										{ ILI9325_R81, 0                                                                                                          },
										{ ILI9325_R82, 0                                                                                                          },
										{ ILI9325_R83, 0                                                                                                          },
										{ ILI9325_R84, 0                                                                                                          },
										{ ILI9325_R85, 0                                                                                                          },
										{ ILI9325_R90, ILI9325_R90_DIVI_FOSC_1|ILI9325_R90_RTNI_16CLK                                                             },
										{ ILI9325_R92, ILI9325_R92_NOWI_6CLK                                                                                      },
										{ ILI9325_R07, ILI9325_R07_PTDE(0)|ILI9325_R07_BASEE|ILI9325_R07_GON|ILI9325_R07_DTE|ILI9325_R07_CL_262K|ILI9325_R07_D(3) },
										{ 0,           0                                                                                                          }
									} ;
