/* ================================================================================ */
/*   driver/mpl115a1.c                                                              */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#include "rtos.h"
#include "mpl115a1.h"

	/* R}h */
	enum {
		CMD_READ_PADC				=	0x80,						/* C A/D ϊlǂݍ */
		CMD_READ_TADC				=	0x84,						/* C A/D ϊlǂݍ */
		CMD_READ_A0					=	0x88,						/* a0  Wǂݍ        */
		CMD_READ_B1					=	0x8C,						/* b1  Wǂݍ        */
		CMD_READ_B2					=	0x90,						/* b2  Wǂݍ        */
		CMD_READ_C12				=	0x94,						/* c12 Wǂݍ        */
		CMD_CONVERT					=	0x24						/* CEC A/D ϊJn */
	} ;


static __NEAR int16_t				m_a0 ;							/* a0  W */
static __NEAR int16_t				m_b1 ;							/* b1  W */
static __NEAR int16_t				m_b2 ;							/* b2  W */
static __NEAR int16_t				m_c12 ;							/* c12 W */


/* -------------------------------------------------------------------------------- */
/*   private                                                                        */

/* ---------------------------------------------------------------- */
/*   wR}h̒lǂݍ                                     */
/*                                                                  */
/*   IN : cmd = R}h                                            */
/*   OUT: l                                                        */
/* ---------------------------------------------------------------- */
uint16_t read( uint8_t cmd )
{
	uint8_t		hi, lo ;


	MPL115A1_XFER( cmd ) ;
	hi = MPL115A1_XFER( 0x00 ) ;

	cmd += 2 ;

	MPL115A1_XFER( cmd ) ;
	lo = MPL115A1_XFER( 0x00 ) ;

	return (((uint16_t )hi << 8) | lo) ;
}




/* -------------------------------------------------------------------------------- */
/*   public                                                                         */

/* ---------------------------------------------------------------- */
/*   Wǂݍ                                                 */
/*                                                                  */
/*   IN : Ȃ                                                      */
/*   OUT: Ȃ                                                      */
/* ---------------------------------------------------------------- */
void mpl115a1_readCoefficient( void )
{
	/* N҂ */
	MPL115A1_WAIT5ms() ;

	/* Wf[^ǂݍ */
	MPL115A1_CS_LO() ;
	m_a0  = read( CMD_READ_A0 ) ;
	m_b1  = read( CMD_READ_B1 ) ;
	m_b2  = read( CMD_READ_B2 ) ;
	m_c12 = read( CMD_READ_C12 ) ;
	MPL115A1_CS_HI() ;

	m_c12 >>= 2 ;
}


/* ---------------------------------------------------------------- */
/*   A/D ϊlǂݍ                                           */
/*                                                                  */
/*   IN : p_adc = A/D ϊli[AhX                            */
/*   OUT: Ȃ                                                      */
/* ---------------------------------------------------------------- */
void mpl115a1_read( MPL115A1ADC MPL115A1_SEG_ADC *p_adc )
{
	MPL115A1_LOCK() ;

	/* ϊJn */
	MPL115A1_CS_LO() ;
	MPL115A1_XFER( CMD_CONVERT ) ;
	MPL115A1_XFER( 0x00 ) ;
	MPL115A1_CS_HI() ;

	/* ϊ҂ */
	MPL115A1_WAIT3ms() ;

	/* A/D ϊlǂݍ */
	MPL115A1_CS_LO() ;
	p_adc->padc = read( CMD_READ_PADC ) >> 6 ;
	p_adc->tadc = read( CMD_READ_TADC ) >> 6 ;
	MPL115A1_CS_HI() ;

#if     (MPL115A1_ADC_COUNT > 1 )
{
	uint8_t		index ;


	for(index=1; index<MPL115A1_ADC_COUNT; index++) {
		MPL115A1_CS_LO() ;
		p_adc->padc += (read( CMD_READ_PADC ) >> 6) ;
		p_adc->tadc += (read( CMD_READ_TADC ) >> 6) ;
		MPL115A1_CS_HI() ;
	}

	p_adc->padc /= MPL115A1_ADC_COUNT ;
	p_adc->tadc /= MPL115A1_ADC_COUNT ;
}
#endif/*(MPL115A1_ADC_COUNT < 2 )*/

	MPL115A1_UNLOCK() ;
}


/* ---------------------------------------------------------------- */
/*   C擾                                                     */
/*                                                                  */
/*   IN : p_adc = A/D ϊl                                        */
/*   OUT: C x 10                                                 */
/* ---------------------------------------------------------------- */
int16_t mpl115a1_getPressure( const MPL115A1ADC MPL115A1_SEG_ADC *p_adc )
{
	int32_t		c12x2, a1, a1x1, y1, a2x2, pcomp ;


	/* m_a0        = siiiiiiiiiiiifff : siiiiiiiiiiii.fff                    */
	/* m_b1        = siifffffffffffff : s          ii.fffffffffffff          */
	/* m_b2        = siffffffffffffff : s           i.ffffffffffffff         */
	/* m_c12       = sssfffffffffffff : s           0.---------fffffffffffff */
	/* p_adc->padc = ------iiiiiiiiii :    iiiiiiiiii                        */
	/* p_adc->tadc = ------iiiiiiiiii :    iiiiiiiiii                        */

	c12x2   = ((int32_t )m_c12 * p_adc->tadc) >> 9 ;
	a1      =  (int32_t )m_b1 + c12x2 ;
	a1x1    = a1 * p_adc->padc ;
	y1      = ((int32_t )m_a0 << 10) + a1x1 ;
	a2x2    = ((int32_t )m_b2 * p_adc->tadc) >> 1 ;
	pcomp   = y1 + a2x2 ;

	return (int16_t )((((pcomp >> 8) * 6500) >> 15) + 5000) ;
}


/* ---------------------------------------------------------------- */
/*   C擾                                                     */
/*                                                                  */
/*   IN : tadc = C A/D ϊl                                  */
/*   OUT: C x 10                                                 */
/* ---------------------------------------------------------------- */
int16_t mpl115a1_getTemperature( uint16_t tadc )
{
	/* C  = (tadc - 472) / -5.35 + 25 */
	/* -5.35 = tadc  1                 */
	/* 472   = 25 x                     */

	return (int16_t )((int32_t )(int )(472 - tadc) * 1000 / 535 + 250 ) ;
}
