/* ================================================================================ */
/*   driver/sdc.c                                                                   */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#include "rtos.h"
#include "sdc.h"

#define sendCmdParam(cmd, param)						\
(														\
	m_cmd[0] = (uint8_t )((uint32_t )(param)>>24),		\
	m_cmd[1] = (uint8_t )((uint32_t )(param)>>16),		\
	m_cmd[2] = (uint8_t )((uint32_t )(param)>> 8),		\
	m_cmd[3] = (uint8_t )((uint32_t )(param)>> 0),		\
	sendCmd( cmd )										\
)


	enum {
		BLOCK_SIZE					=	512							/* ubNTCY */
	} ;

	/* R}ht */
	enum {
		CMD_ARG						=	0x8000,						/*  */
		CMD_R1						=	0 << 8,						/* Resp R1  */
		CMD_R1b						=	0 << 8,						/* Resp R1b */
		CMD_R2						=	1 << 8,						/* Resp R2  */
		CMD_R3						=	4 << 8,						/* Resp R3  */
		CMD_R7						=	4 << 8						/* Resp R7  */
	} ;

	/* R}h */
	enum {
		GO_IDLE_STATE				=	          CMD_R1  | 0x40 |  0,
		SEND_OP_COND				=	          CMD_R1  | 0x40 |  1,
		SEND_IF_COND				=	CMD_ARG | CMD_R7  | 0x40 |  8,
		SEND_CSD					=	          CMD_R1  | 0x40 |  9,
		SEND_CID					=	          CMD_R1  | 0x40 | 10,
		STOP_TRANSMISSION			=	          CMD_R1  | 0x40 | 12,
		SEND_STATUS					=	          CMD_R2  | 0x40 | 13,
		SET_BLOCKLEN				=	CMD_ARG | CMD_R1  | 0x40 | 16,
		READ_SINGLE_BLOCK			=	CMD_ARG | CMD_R1  | 0x40 | 17,
		READ_MULTIPLE_BLOCK			=	CMD_ARG | CMD_R1  | 0x40 | 18,
		SET_BLOCK_COUNT				=	CMD_ARG | CMD_R1  | 0x40 | 23,
		WRITE_BLOCK					=	CMD_ARG | CMD_R1  | 0x40 | 24,
		WRITE_MULTIPLE_BLOCK		=	CMD_ARG | CMD_R1  | 0x40 | 25,
		PROGRAM_CSD					=	          CMD_R1b | 0x40 | 27,
		SET_WRITE_PROT				=	CMD_ARG | CMD_R1b | 0x40 | 28,
		CLR_WRITE_PROT				=	CMD_ARG | CMD_R1b | 0x40 | 29,
		SEND_WRITE_PROT				=	CMD_ARG | CMD_R1  | 0x40 | 30,
		TAG_SECTOR_START			=	CMD_ARG | CMD_R1  | 0x40 | 32,
		TAG_SECTOR_END				=	CMD_ARG | CMD_R1  | 0x40 | 33,
		UNTAG_SECTOR				=	CMD_ARG | CMD_R1  | 0x40 | 34,
		TAG_ERASE_GROUP_START		=	CMD_ARG | CMD_R1  | 0x40 | 35,
		TAG_ERASE_GROUP_END			=	CMD_ARG | CMD_R1  | 0x40 | 36,
		UNTAG_ERASE_GROUP			=	CMD_ARG | CMD_R1  | 0x40 | 37,
		ERASE						=	CMD_ARG | CMD_R1  | 0x40 | 38,
		APP_SEND_OP_COND			=	CMD_ARG | CMD_R1  | 0x40 | 41,
		LOCK_UNLOCK					=	CMD_ARG | CMD_R1b | 0x40 | 42,
		APP_CMD						=	          CMD_R1  | 0x40 | 55,
		READ_OCR					=	          CMD_R3  | 0x40 | 58,
		CRC_ON_OFF					=	CMD_ARG | CMD_R1  | 0x40 | 59
	} ;


static BOOL							m_blockAddress ;				/* ubNEAhbVO */
static __NEAR uint8_t				m_cmd[4] ;						/* R}hiE߂lj */


/* ------------------------------------------------------------------------------------------------ */
/*   private                                                                                        */

/* -------------------------------------------------------------------------------- */
/*   rW[`FbN                                                                 */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: E_OK                                                                      */
/*        E_TMOUT                                                                   */
/* -------------------------------------------------------------------------------- */
int8_t checkBusy( void )
{
	uint8_t		retry ;


	retry = 100 ;

	do {
		if( SDC_XFER( 0xff ) == 0xff ) {
			return E_OK ;
		}
	} while( --retry ) ;

	retry = 100 ;

	do {
		if( SDC_XFER( 0xff ) == 0xff ) {
			return E_OK ;
		}
		dly_tsk( 10 ) ;
	} while( --retry ) ;

	return E_TMOUT ;
}


/* -------------------------------------------------------------------------------- */
/*   R}h𑗐M                                                                 */
/*                                                                                  */
/*   IN : cmd = R}h                                                            */
/*   OUT: X|X                                                                */
/*        E_TMOUT                                                                   */
/* -------------------------------------------------------------------------------- */
int8_t sendCmd( uint16_t cmd )
{
	int8_t		result ;
	uint8_t		count ;

	/* rW[`FbN */
	result = checkBusy() ;
	if( result ) {
		return result ;
	}

	/* R}hM */
	SDC_XFER( (uint8_t )cmd ) ;
	if( cmd & CMD_ARG ) {	/*  */
		SDC_XFER( m_cmd[0] ) ;
		SDC_XFER( m_cmd[1] ) ;
		SDC_XFER( m_cmd[2] ) ;
		SDC_XFER( m_cmd[3] ) ;
	} else {				/* Ȃ */
		SDC_XFER( 0x00 ) ;
		SDC_XFER( 0x00 ) ;
		SDC_XFER( 0x00 ) ;
		SDC_XFER( 0x00 ) ;
	}

	/* CRC */
	if( (uint8_t )cmd == 0x40 ) {
		SDC_XFER( 0x95 ) ;
	} else
	if( (uint8_t )cmd == 0x48 ) {
		SDC_XFER( 0x87 ) ;
	} else {
		SDC_XFER( 0xff ) ;
	}

	/* X|X҂ */
	count = 8 + 1 ;
	while( 1 ) {
		result = SDC_XFER( 0xff ) ;
		if( !(result&0x80) ) {
			break ;
		}

		if( !--count ) {
			return E_TMOUT ;
		}
	}

	/* ߂l */
	count = (uint8_t )(cmd >> 8) & 0x0f ;
	if( count == 1 ) {
		m_cmd[0] = SDC_XFER( 0xff ) ;
	} else
	if( count == 4 ) {
		m_cmd[0] = SDC_XFER( 0xff ) ;
		m_cmd[1] = SDC_XFER( 0xff ) ;
		m_cmd[2] = SDC_XFER( 0xff ) ;
		m_cmd[3] = SDC_XFER( 0xff ) ;
	}

	return result ;
}




/* ------------------------------------------------------------------------------------------------ */
/*   public                                                                                         */

/* -------------------------------------------------------------------------------- */
/*   Zbg                                                                       */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: E_OK                                                                      */
/*        E_SYS                                                                     */
/*        E_TMOUT                                                                   */
/* -------------------------------------------------------------------------------- */
int8_t sdc_reset( void )
{
	int8_t		result ;
	uint8_t		index ;


	SDC_LOCK() ;

	SDC_XFER( 0xff ) ;

	dly_tsk( 1 ) ;

	SDC_CS_HI() ;

	/* R}h󂯕tiVANbN 74clk ȏój */
	for(index=0; index<10; index++) {
		SDC_XFER( 0xff ) ;
	}

	m_blockAddress = 0 ;

	SDC_CS_LO() ;

	do {
		/* Zbg */
		result = sendCmd( GO_IDLE_STATE ) ;
		if( result != 0x01 ) {
			result |= 1 ;
			break ;
		}

		/* J[hގ擾 */
		result = sendCmdParam(SEND_IF_COND, 0x0001aa) ;
		if( result == 0x01 ) {			/* SDC V2 */

			/* ߂l 0x01aa łȂȂgps */
			if( (m_cmd[2]!=0x01) || (m_cmd[3]!=0xaa) ) {
				result = E_SYS ;
				break ;
			}

// puts( "SDC V2\r\n") ;

			/*  */
			for(index=0; index<1000/10; index++) {
				result = sendCmd( APP_CMD ) ;
				if( result == 0x01 ) {
					result = sendCmdParam(APP_SEND_OP_COND, 0x40000000) ;
					if( !result ) {
						break ;
					}
				}
				dly_tsk( 10 ) ;
			}
			if( result ) {
				break ;
			}

			/* OCR ǂݍ */
			result = sendCmd( READ_OCR ) ;
			if( result ) {
				break ;
			}

			/* ubNEAhbVO */
			if( m_cmd[1] & 0x40 ) {
				m_blockAddress = 1 ;
			}

		} else
		if( (result&0x84) == 0x04 ) {	/* SDC V1 / MMC */

// puts( "SDC V1 / MMC\r\n") ;

			/*  */
			for(index=0; index<1000/10; index++) {
				result = sendCmd( SEND_OP_COND ) ;
				if( !result ) {
					break ;
				}
				dly_tsk( 10 ) ;
			}
			if( result ) {
				break ;
			}

		} else {
			result = E_SYS ;
			break ;
		}

		/* ubNTCYݒ */
		if( !m_blockAddress ) {
			result = sendCmdParam(SET_BLOCKLEN, BLOCK_SIZE) ;
		}
	} while( 0 ) ;

	SDC_CS_HI() ;

	SDC_XFER( 0xff ) ;

	SDC_UNLOCK() ;

	if( result > 0 ) {
		return E_SYS ;
	}

	return result ;
}


/* -------------------------------------------------------------------------------- */
/*   wubNǂݍ                                                         */
/*                                                                                  */
/*   IN : address = ubNAhX                                                */
/*        p_data  = ǂݍ݃f[^i[obt@                                      */
/*        block   = ǂݍ݃ubN                                              */
/*   OUT: E_OK                                                                      */
/*        E_SYS                                                                     */
/*        E_TMOUT                                                                   */
/* -------------------------------------------------------------------------------- */
int8_t sdc_read(uint32_t address, uint8_t __FAR *p_data, uint8_t block)
{
	int8_t		result ;
	uint8_t		count, size ;


	SDC_LOCK() ;

	/* ubNEoCgϊ */
	if( !m_blockAddress ) {
		address *= BLOCK_SIZE ;
	}

	SDC_CS_LO() ;

	while( 1 ) {
		/* VOEubNǂݏo */
		result = sendCmdParam(READ_SINGLE_BLOCK, address) ;
		if( result ) {
			break ;
		}

		/* f[^EpPbg҂ */
		count = 200 ;
		while( 1 ) {
			result = SDC_XFER( 0xff ) ;
			if( (uint8_t )result == 0xfe ) {
				break ;
			}

			/* G[Eg[N */
			if( !(result&0xe0) ) {
				result = E_SYS ;
				goto sdc_read_fff ;
			}

			/* ^CAEg */
			if( !--count ) {
				result = E_TMOUT ;
				goto sdc_read_fff ;
			}
		}

		/* f[^EubN */
		size = 0 ;
		do {
			*p_data++ = SDC_XFER( 0xff ) ;
			*p_data++ = SDC_XFER( 0xff ) ;
		} while( --size ) ;

		/* CRC */
		SDC_XFER( 0xff ) ;
		SDC_XFER( 0xff ) ;

		if( !--block ) {
			result = E_OK ;
			break ;
		}

		if( !m_blockAddress ) {
			address += BLOCK_SIZE ;
		} else {
			address++ ;
		}
	}

sdc_read_fff:
	SDC_CS_HI() ;

	SDC_XFER( 0xff ) ;

	SDC_UNLOCK() ; ;

	if( result > 0 ) {
		return E_SYS ;
	}

	return result ;
}


/* -------------------------------------------------------------------------------- */
/*   wubN֏                                                         */
/*                                                                                  */
/*   IN : address = ubNAhX                                                */
/*        p_data  = ݃f[^                                                  */
/*        block   = ݃ubN                                              */
/*   OUT: E_OK                                                                      */
/*        E_SYS                                                                     */
/*        E_TMOUT                                                                   */
/* -------------------------------------------------------------------------------- */
int8_t sdc_write(uint32_t address, const uint8_t __FAR *p_data, uint8_t block)
{
	int8_t		result ;
	uint8_t		size ;


	SDC_LOCK() ;

	/* ubNEoCgϊ */
	if( !m_blockAddress ) {
		address *= BLOCK_SIZE ;
	}

	SDC_CS_LO() ;

	while( 1 ) {
		/* VOEubN */
		result = sendCmdParam(WRITE_BLOCK, address) ;
		if( result ) {
			break ;
		}

		/* _~[ */
		SDC_XFER( 0xff ) ;

		/* f[^Eg[N */
		SDC_XFER( 0xfe ) ;

		/* f[^EubN */
		size = 0 ;
		do {
			SDC_XFER( *p_data ) ;
			p_data++ ;
			SDC_XFER( *p_data ) ;
			p_data++ ;
		} while( --size ) ;

		/* CRC */
		SDC_XFER( 0xff ) ;
		SDC_XFER( 0xff ) ;

		/* f[^X|X */
		result = SDC_XFER( 0xff ) & 0x1f ;
		if( result != 0x05 ) {
			result = E_SYS ;
			break ;
		}

		/* _~[ */
		SDC_XFER( 0xff ) ;

		/* rW[`FbN */
		result = checkBusy() ;
		if( result ) {
			break ;
		}

		if( !--block ) {
			result = E_OK ;
			break ;
		}

		if( !m_blockAddress ) {
			address += BLOCK_SIZE ;
		} else {
			address++ ;
		}
	}

	SDC_CS_HI() ;

	SDC_XFER( 0xff ) ;

	SDC_UNLOCK() ;

	if( result > 0 ) {
		return E_SYS ;
	}

	return result ;
}
