/* ================================================================================ */
/*   driver/usb.h                                                                   */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#ifndef __DRIVER__USB_H__INCLUDED__
#define __DRIVER__USB_H__INCLUDED__


	/* SETUP R}h */
	enum {
		SC_GET_STATUS				=	0x00,						/* Get Status            */
			GS_DEVICE				=	0x80,						/*   Device              */
			GS_INTERFACE			=	0x81,						/*   Interface           */
			GS_ENDPOINT				=	0x82,						/*   End Point           */
		SC_CLEAR_FEATURE			=	0x01,						/* Clear Feature         */
			FT_DEVICE				=	0x00,						/*   Device              */
			FT_ENDPOINT				=	0x02,						/*   End Point           */
		SC_RESERVED					=	0x02,						/* Reserved              */
		SC_SET_FEATURE				=	0x03,						/* Set Feature           */
		SC_SET_ADDRESS				=	0x05,						/* Set Address           */
		SC_GET_DESCRIPTOR			=	0x06,						/* Get Descriptor        */
			GD_DEVICE				=	0x01,						/*   Device              */
			GD_CONFIGURATION		=	0x02,						/*   Configuration       */
			GD_STRING				=	0x03,						/*   String              */
			GD_INTERFACE			=	0x04,						/*   Interface           */
			GD_ENDPOINT				=	0x05,						/*   Endpoint            */
			GD_DEVICE_QUALIFIER		=	0x06,						/*   Device Qualifier    */
			GD_OTHER_SPEED_CONFIGURATION	=	0x07,				/*   Other Configuration */
			GD_INTERFACE_POWER		=	0x08,						/*   Interface Power     */
			GD_HID					=	0x21,						/*   HID                 */
			GD_REPORT				=	0x22,						/*   Report              */
		SC_SET_DESCRIPTOR			=	0x07,						/* Set Descriptor        */
		SC_GET_CONFIGURATION		=	0x08,						/* Get Configuration     */
		SC_SET_CONFIGURATION		=	0x09,						/* Set Configuration     */
		SC_GET_INTERFACE			=	0x0a,						/* Get Interface         */
		SC_SET_INTERFACE			=	0x0b,						/* Set Interface         */
		SC_SYNC_FRAME				=	0x0c,						/* Sync Frame            */
		SC_ANCHOR_LOAD				=	0xa0
	} ;

	/* fBXNv^ */
	enum {
		DSCR_DEVICE					=	0x01,						/* foCX                       */
		DSCR_CONFIG					=	0x02,						/* RtBM[V         */
		DSCR_STRING					=	0x03,						/* XgO                     */
		DSCR_INTERFACE				=	0x04,						/* C^tF[X                 */
		DSCR_ENDPOINT				=	0x05,						/* Gh|Cg                 */
		DSCR_DEVICEQUALIFIER		=	0x06,						/* foCXNHt@CA         */
		DSCR_OTHERSPEEDCONFIG		=	0x07,						/*                                */
		DSCR_INTERFACEPOWER			=	0x08,						/*                                */
		DSCR_OTG					=	0x09,						/* OTG                            */
		DSCR_DEBUG					=	0x0a,						/* fobO                       */
		DSCR_INTERFACEASSOCIATION	=	0x0b,						/* C^tF[XA\VG[V */
		DSCR_HID					=	0x21,						/* HID                            */
		DSCR_HIDREPORT				=	0x22,						/* HID |[g                   */
		DSCR_CSINTERFACE			=	0x24						/* NXŗL                     */
	} ;

	/* C^tF[XNXR[h */
	enum {
		NOSUBCLASS					=	0x00,						/* TuNXȂ */
		BOOTSUBCLASS				=	0x01,						/*  */
		HIDCLASS					=	0x03,						/* HID NX     */
		VENDORSPECCLASS				=	0xff						/* x_L     */
	} ;

	/* Gh|Cg */
	enum {
		EP_CONTROL					=	0x00,						/* Rg[]   */
		EP_ISO						=	0x01,						/* AC\NiX] */
		EP_BULK						=	0x02,						/* oN]         */
		EP_INT						=	0x03						/* C^vg]   */
	} ;

	/* ZbgAbvf[^ */
//	typedef struct {
//		uint8_t						bmRequestType ;					/* NGXg^Cv               */
//		uint8_t						bRequest ;						/* NGXg                     */
//		uint16_t					wValue ;						/* foCX                   */
//		uint16_t					wIndex ;						/* foCX                   */
//		uint16_t					wLength ;						/* Ղɑf[^Xe[WoCg */
//	} SETUPDAT ;

	/* foCXfBXNv^ */
	typedef struct {
		uint8_t						bLength ;						/* fBXNv^                           */
		uint8_t						bDescriptorType ;				/* fBXNv^                         */
		uint16_t					bcdUSB ;						/* USB dl[XԍiBCDj              */
		uint8_t						bDeviceClass ;					/* NXR[h                               */
		uint8_t						bDeviceSubclass ;				/* TuNXR[h                           */
		uint8_t						bDeviceProtocol ;				/* vgRR[h                           */
		uint8_t						bMaxPacketSize0 ;				/* Gh|CgO̍őpPbgTCY       */
		uint16_t					idVendor ;						/* x_ʎq                               */
		uint16_t					idProduct ;						/* v_Ngʎq                           */
		uint16_t					bcdDevice ;						/* foCX[XԍiBCDj                */
		uint8_t						iManufacturer ;					/* Ǝ҃XgOCfbNX             */
		uint8_t						iProduct ;						/* v_NgXgOCfbNX           */
		uint8_t						iSerialNumber ;					/* foCXVAԍXgOCfbNX */
		uint8_t						bNumConfigurations ;			/* RtBM[V                   */
	} DEVICEDSCR ;

	/* RtBM[VfBXNv^ */
	typedef struct {
		uint8_t						bLength ;						/* fBXNv^                             */
		uint8_t						bDescriptorType ;				/* fBXNv^                           */
		uint16_t					wTotalLength ;					/* ݂я]fBXNv^               */
		uint8_t						bNumInterfaces ;				/* C^tF[X                             */
		uint8_t						bConfigurationValue ;			/* RtBM[VCfbNX           */
		uint8_t						iConfiguration ;				/* RtBM[VXgOCfbNX */
		uint8_t						bmAttributes ;					/*                                          */
		uint8_t						bMaxPower ;						/* ődimAj/ 2                        */
	} CONFIGDSCR ;

	/* C^tF[XfBXNv^ */
	typedef struct {
		uint8_t						bLength ;						/* fBXNv^                     */
		uint8_t						bDescriptorType ;				/* fBXNv^                   */
		uint8_t						bInterfaceNumber ;				/* C^tF[Xԍ                   */
		uint8_t						bAlternateSetting ;				/* I^l[gݒԍ                 */
		uint8_t						bNumEndpoints ;					/* ̃C^tF[X̃Gh|Cg */
		uint8_t						bInterfaceClass ;				/* C^tF[XNX                 */
		uint8_t						bInterfaceSubClass ;			/* C^tF[XTuNX             */
		uint8_t						bInterfaceProtocol ;			/* C^tF[XvgR             */
		uint8_t						iInterface ;					/* C^tF[XXgOCfbNX */
	} INTERFACEDSCR ;

	/* Gh|CgfBXNv^ */
	typedef struct {
		uint8_t						bLength ;						/* fBXNv^         */
		uint8_t						bDescriptorType ;				/* fBXNv^       */
		uint8_t						bEndpointAddress ;				/* Gh|Cgԍƕ */
		uint8_t						bmAttributes ;					/* ]                 */
		uint16_t					wMaxPacketSize ;				/* őpPbgTCY       */
		uint8_t						bInterval ;						/* |[O           */
	} ENDPOINTDSCR ;

	/* XgOfBXNv^ */
	typedef struct {
		uint8_t						bLength ;						/* fBXNv^ */
		uint8_t						bDescriptorType ;				/* fBXNv^ */
		union {
			uint16_t				wLANGID[1] ;					/* ꎯʃR[h   */
			uint16_t				bSTRING[1] ;					/*            */
		} ;
	} STRINGDSCR ;

	/* HID NXfBXNv^ */
	typedef struct {
		uint8_t						bLength ;						/* fBXNv^            */
		uint8_t						bDescriptorType ;				/* fBXNv^          */
		uint16_t					bcdHID ;						/* HID dl[XԍiBCDj */
		uint8_t						bCountryCode ;					/* ʃR[h                  */
		uint8_t						bNumDescriptors ;				/* ]NXfBXNv^  */
		uint8_t						bClassDescriptorType ;			/* fBXNv^          */
		uint16_t					wDescriptorLength ;				/* |[gfBXNv^    */
	} HIDDSCR ;


#endif/*__DRIVER__USB_H__INCLUDED__*/
