/* ================================================================================ */
/*   ezusb_fx2/i2c.c                                                                */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#include "rtos.h"

	/* 샂[h */
	enum {
		MODE_SENDDATA,										/* f[^M                       */
		MODE_RECV,											/* X[uAhXM̎M҂ */
		MODE_RECVDATA,										/* f[^M                       */
		MODE_RECVEND,										/* f[^MI                   */
		MODE_COMP,											/* M                       */
		MODE_ERROR											/* G[                           */
	} ;


static __data int8_t				m_ercd ;				/* G[R[h */
static __data uint8_t				m_mode ;				/* 샂[h   */
static __data uint8_t				m_flag ;				/* tO       */
static __data uint8_t				m_count ;				/* f[^     */
static __xdata uint8_t 				* __data m_p_data ;		/* f[^       */


/* -------------------------------------------------------------------------------- */
/*   M                                                                           */
/*                                                                                  */
/*   IN : saddr  = X[uAhX                                                 */
/*        p_data = f[^i[AhX                                               */
/*        count  = f[^                                                         */
/*   OUT: E_OK                                                                      */
/*        E_TMOUT                                                                   */
/*        I2C_E_BUS                                                                 */
/*        I2C_E_NACK                                                                */
/* -------------------------------------------------------------------------------- */
int8_t i2c_send(uint8_t addr, const void __xdata *p_data, uint8_t count)
{
	int8_t	result ;

	/*  */
	m_ercd   = E_OK ;
	m_flag   = 0 ;
	m_count  = count ;
	m_p_data = (__xdata uint8_t * __data)p_data ;
	m_mode   = MODE_SENDDATA ;

	/* ]Jn */
	I2CS  = I2CS_START ;
	I2DAT = (addr << 1) ;

	/* ]҂ */
	result = twai_sem(I2C_SEMID, I2C_TMOUT) ;
	if( result ) {
		I2CS |= I2CS_STOP ;
		return E_TMOUT ;
	}

	return m_ercd ;
}


/* -------------------------------------------------------------------------------- */
/*   M                                                                           */
/*                                                                                  */
/*   IN : saddr  = X[uAhX                                                 */
/*        p_data = f[^i[AhX                                               */
/*        count  = f[^                                                         */
/*   OUT: E_OK                                                                      */
/*        E_TMOUT                                                                   */
/*        I2C_E_BUS                                                                 */
/*        I2C_E_NACK                                                                */
/* -------------------------------------------------------------------------------- */
int8_t i2c_recv(uint8_t saddr, void __xdata *p_data, uint8_t count)
{
	int8_t	result ;

	/*  */
	m_ercd   = E_OK ;
	m_flag   = I2CINFO_FLAG_READ ;
	m_count  = count ;
	m_p_data = p_data ;
	m_mode   = MODE_RECV ;

	/* ]Jn */
	I2CS  = I2CS_START ;
	I2DAT = (saddr << 1) | 1 ;

	/* ]҂ */
	result = twai_sem(I2C_SEMID, I2C_TMOUT) ;
	if( result ) {
		I2CS |= I2CS_STOP ;
		return E_TMOUT ;
	}

	return m_ercd ;
}


/* -------------------------------------------------------------------------------- */
/*   ]                                                                           */
/*                                                                                  */
/*   IN : p_i2cInfo = I2C                                                       */
/*        count     = I2C                                                     */
/*   OUT: E_OK                                                                      */
/*        E_PAR                                                                     */
/*        E_TMOUT                                                                   */
/*        I2C_E_BUS                                                                 */
/*        I2C_E_NACK                                                                */
/* -------------------------------------------------------------------------------- */
int8_t i2c_xfer(const I2CINFO __xdata *p_i2cInfo, uint8_t count)
{
	int8_t	result ;


	if( !count ) {
		return E_PAR ;
	}

	m_ercd = E_OK ;

	while( 1 ) {
		/*  */
		m_flag   = p_i2cInfo->flag ;
		m_count  = p_i2cInfo->count ;
		m_p_data = (__xdata uint8_t * __data)p_i2cInfo->p_data ;

		if( m_flag & I2CINFO_FLAG_NOADDR ) {	/* X[uAhXMȂ */

			/* MȂI */
			if( p_i2cInfo->saddr & I2CINFO_FLAG_READ ) {
				return E_PAR ;
			}

			/* ]Jn */
			m_mode = MODE_SENDDATA ;
			I2DAT  = *m_p_data++ ;
			m_count-- ;

		} else {								/* X[uAhXM */

			/* ]Jn */
			I2CS = I2CS_START ;

			if( m_flag & I2CINFO_FLAG_READ ) {		/* M */
				m_mode = MODE_RECV ;
				I2DAT  = (p_i2cInfo->saddr << 1) | 1 ;
			} else {								/* M */
				m_mode = MODE_SENDDATA ;
				I2DAT  = (p_i2cInfo->saddr << 1) ;
			}

		}

		/* ]҂ */
		result = twai_sem(I2C_SEMID, I2C_TMOUT) ;
		if( result ) {
			I2CS |= I2CS_STOP ;
			return E_TMOUT ;
		}

		/* MG[ */
		if( m_ercd != E_OK ) {
			return m_ercd ;
		}

		if( !--count ) {
			break ;
		}

		p_i2cInfo++ ;
	}

	return E_OK ;
}


/* -------------------------------------------------------------------------------- */
/*   荞݃T[rX[`                                                       */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void i2c_isr( void ) INTPROC
{
	do {
		/* oXG[ */
		if( I2CS & I2CS_BERR ) {
			m_ercd = I2C_E_BUS ;
			goto isr_i2c_error ;
		}

		switch( m_mode ) {
			case MODE_SENDDATA:
			{
				/* ACK Ȃ */
				if( !(I2CS&I2CS_ACK) ) {
					m_ercd = I2C_E_NACK ;
					goto isr_i2c_error ;
				}

				/* M */
				if( !m_count ) {
					if( !(m_flag&I2CINFO_FLAG_NOSTOP) ) {
						I2CS |= I2CS_STOP ;
					}
					m_mode = MODE_COMP ;
					isig_sem( I2C_SEMID ) ;
					break ;
				}

				I2DAT = *m_p_data++ ;
				m_count-- ;
				break ;
			}

			case MODE_RECV:
			{
				/* ACK Ȃ */
				if( !(I2CS&I2CS_ACK) ) {
					m_ercd = I2C_E_NACK ;
					goto isr_i2c_error ;
				}

				I2DAT ;

				m_mode = MODE_RECVDATA ;

				/* Ō */
				if( !--m_count ) {
					I2CS  |= I2CS_LASTRD ;
					m_mode = MODE_RECVEND ;
				}
				break ;
			}

			case MODE_RECVDATA:
			{
				/* Ō */
				if( !--m_count ) {
					I2CS  |= I2CS_LASTRD ;
					m_mode = MODE_RECVEND ;
				}

				*m_p_data++ = I2DAT ;
				break ;
			}

			case MODE_RECVEND:
			{
				if( !(m_flag&I2CINFO_FLAG_NOSTOP) ) {
					I2CS |= I2CS_STOP ;
				}
				m_mode    = MODE_COMP ;
				*m_p_data = I2DAT ;
				isig_sem( I2C_SEMID ) ;
				break ;
			}

			case MODE_COMP:
			case MODE_ERROR:
			{
				break ;
			}
		}
	} while( 0 ) ;

	EXIF &= ~EXIF_I2CINT ;

	return ;


isr_i2c_error:
	m_mode = MODE_ERROR ;
	I2CS  |= I2CS_STOP ;
	isig_sem( I2C_SEMID ) ;

	EXIF &= ~EXIF_I2CINT ;
}
