/* ================================================================================ */
/*   ezusb_fx2/usb.c                                                                */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#include "rtos.h"
#include "usb.h"

#define epcs( ep )					(volatile uint8_t __xdata *)(0xe6a1+m_epcsOffset[(ep&0x7e)|(ep>>7)])


extern const uint8_t				g_usbDeviceDscr[] ;				/* usb_dscr.asm */
extern const uint8_t				g_usbDeviceQualifierDscr[] ;	/*              */
extern const uint8_t				g_usbHidReportDscr[42] ;		/*              */
extern const uint8_t				g_usbHighSpeedConfigDscr[] ;	/*              */
extern const uint8_t				g_usbFullSpeedConfigDscr[] ;	/*              */
extern const uint16_t				g_usbStringDscr[4] ;			/*              */


static const uint8_t				m_epcsOffset[]={
										0,							/* EP0CS    */
										1,							/* EP1OUTCS */
										2,							/* EP1INCS  */
										2,							/* EP1INCS  */
										3,							/* EP4CS    */
										3,							/* EP4CS    */
										4,							/* EP6CS    */
										4,							/* EP6CS    */
										5,							/* EP8CS    */
										5,							/* EP8CS    */
									} ;


static __data  uint8_t				m_interface ;
static __xdata uint16_t				m_configDscr ;
static __xdata uint16_t				m_otherConfigDscr ;


/* -------------------------------------------------------------------------------- */
/*   obt@̓eGh|CgOőM                                         */
/*                                                                                  */
/*   IN : p_buff = Mobt@                                                     */
/*        length = Mobt@                                                   */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
static void sendBuffer(const uint8_t __code *p_buff, uint8_t length)
{
	uint8_t		index, count ;


	while( length ) {
		/* Rs[TCY */
		count = length ;
		if( count > sizeof( EP0BUF ) ) {
			count = sizeof( EP0BUF ) ;
		}
		length -= count ;

		/* Rs[ */
		for(index=0; index<count; index++) {
			EP0BUF[index] = *p_buff++ ;
		}

		EP0BCH = 0 ;
		EP0BCL = count ;

		/* M҂ */
		while( EP0CS & EP0CS_BUSY ) {
			;
		}
	}
}


/* -------------------------------------------------------------------------------- */
/*   ؒfEĐڑ                                                                   */
/*                                                                                  */
/*   IN : renum = 0:Gj[VȂ                                          */
/*                1:Gj[V                                          */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void disconnect( __bit renum )
{
	if( renum ) {
		USBCS |=(USBCS_DISCON |  USBCS_RENUM) ;
	} else {
		USBCS |= USBCS_DISCON ;
	}

	dly_tsk( 1500 ) ;

	USBIRQ = USBIRQ_EP0ACK | USBIRQ_HSGRANT | USBIRQ_URES | USBIRQ_SUSP | USBIRQ_SUTOK | USBIRQ_SOF | USBIRQ_SUDAV ;
	EPIRQ  = EPIRQ_EP8 | EPIRQ_EP6 | EPIRQ_EP4 | EPIRQ_EP2 | EPIRQ_EP1OUT | EPIRQ_EP1IN | EPIRQ_EP0OUT | EPIRQ_EP0IN ;
	EXIF  &= ~EXIF_USBINT ;

	/* Đڑ */
	USBCS &= ~USBCS_DISCON ;
}


/* -------------------------------------------------------------------------------- */
/*   foCXEC^tF[XEGh|Cg̋@\̏Ԃ擾                     */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: 0 = I                                                              */
/*        1 = Xg[                                                              */
/* -------------------------------------------------------------------------------- */
static __bit getStatus( void )
{
	switch( SETUPDAT[0] ) {
		case GS_DEVICE:
		case GS_INTERFACE:
		{
			EP0BUF[0] = 0x00 ;
			break ;
		}

		case GS_ENDPOINT:
		{
			EP0BUF[0] = *epcs( SETUPDAT[4] ) & EPnCS_STALL ;
			break ;
		}

		default:
			return 1 ;
	}

	EP0BUF[1] = 0x00 ;
	EP0BCH    = 0 ;
	EP0BCL    = 2 ;

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   foCXEC^tF[XEGh|Cg̋@\𖳌                           */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: 0 = I                                                              */
/*        1 = Xg[                                                              */
/* -------------------------------------------------------------------------------- */
static __bit clearFeature( void )
{
	switch( SETUPDAT[0] ) {
		case FT_DEVICE:
		{
			if( SETUPDAT[2] != 1 ) {
				return 1 ;
			}
			break ;
		}

		case FT_ENDPOINT:
		{
			if( SETUPDAT[2] ) {
				return 1 ;
			}

			*epcs( SETUPDAT[4] ) &= ~EPnCS_STALL ;
			TOGCTL                = (((SETUPDAT[4] & TOGCTL_Q) >> 3) | (SETUPDAT[4] & (TOGCTL_EP3|TOGCTL_EP2|TOGCTL_EP1|TOGCTL_EP0))) ;
			break ;
		}

		default:
			return 1 ;
	}

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   foCXEC^tF[XEGh|Cg̋@\ݒ                           */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: 0 = I                                                              */
/*        1 = Xg[                                                              */
/* -------------------------------------------------------------------------------- */
static __bit setFeature( void )
{
	switch( SETUPDAT[0] ) {
		case FT_DEVICE:
		{
			if( (SETUPDAT[2]!=1) && (SETUPDAT[2]!=2) ) {
				return 1 ;
			}
			break ;
		}

		case FT_ENDPOINT:
		{
			*epcs( SETUPDAT[4] ) |= EPnCS_STALL ;
			break ;
		}

		default:
			return 1 ;
	}

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/*   fBXNv^擾                                                           */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: 0 = I                                                              */
/*        1 = Xg[                                                              */
/* -------------------------------------------------------------------------------- */
static __bit getDescriptor( void )
{
	switch( SETUPDAT[3] ) {
		case GD_DEVICE:
		{
			if( SETUPDAT[2] ) {
				return 1 ;
			}

			SUDPTRH = MSB( g_usbDeviceDscr ) ;
			SUDPTRL = LSB( g_usbDeviceDscr ) ;
			break ;
		}

		case GD_CONFIGURATION:
		{
			if( SETUPDAT[2] ) {
				return 1 ;
			}

			SUDPTRH = MSB( m_configDscr ) ;
			SUDPTRL = LSB( m_configDscr ) ;
			break ;
		}

		case GD_STRING:
		{
			uint8_t		index ;
			uint16_t	stringDscr ;


			index = SETUPDAT[2] ;
			if( index >= (sizeof( g_usbStringDscr )/sizeof( g_usbStringDscr[0] )) ) {
				return 1 ;
			}

			stringDscr = g_usbStringDscr[index] ;
			SUDPTRH    = MSB( stringDscr ) ;
			SUDPTRL    = LSB( stringDscr ) ;
			break ;
		}

		case GD_DEVICE_QUALIFIER:
		{
			SUDPTRH = MSB( g_usbDeviceQualifierDscr ) ;
			SUDPTRL = LSB( g_usbDeviceQualifierDscr ) ;
			break ;
		}

		case GD_OTHER_SPEED_CONFIGURATION:
		{
			SUDPTRH = MSB( m_otherConfigDscr ) ;
			SUDPTRL = LSB( m_otherConfigDscr ) ;
			break ;
		}

		case GD_REPORT:
		{
			sendBuffer(&g_usbHidReportDscr[0], sizeof( g_usbHidReportDscr )) ;

			/* eXgp^C}NiƂ肠ŋNj */
			sta_cyc( USB_CYCID ) ;
			break ;
		}

		default:
			return 1 ;
	}

	return 0 ;
}


/* -------------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------------- */
static void delay( void )
{
	nop() ;
	nop() ;
	nop() ;
}


/* -------------------------------------------------------------------------------- */
/*   USB ^XN                                                                     */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_tsk( void )
{
	__bit	stall ;

	/* fBXNv^擾 */
#if 1
	if( USBCS & USBCS_HSM ) {
		m_configDscr      = (uint16_t )&g_usbHighSpeedConfigDscr[0] ;
		m_otherConfigDscr = (uint16_t )&g_usbFullSpeedConfigDscr[0] ;
	} else {
		m_configDscr      = (uint16_t )&g_usbFullSpeedConfigDscr[0] ;
		m_otherConfigDscr = (uint16_t )&g_usbHighSpeedConfigDscr[0] ;
	}
#else
	m_configDscr      = (uint16_t )&g_usbFullSpeedConfigDscr[0] ;
	m_otherConfigDscr = (uint16_t )&g_usbFullSpeedConfigDscr[0] ;
#endif

	loc_cpu() ;

	/* Gh|CgRtBO */
	EP1OUTCFG = EP1OUTCFG_VALID_ACTIVE | EP1OUTCFG_TYPE_BULK ;
	EP1INCFG  = EP1INCFG_VALID_ACTIVE  | EP1INCFG_TYPE_INTERRUPT ;

	delay() ;  EP2CFG = EP2CFG_VALID_ACTIVE | EP2CFG_DIR_OUT | EP2CFG_TYPE_BULK | EP2CFG_SIZE_512 | EP2CFG_BUF_DOUBLE ;
	delay() ;  EP4CFG = EP4CFG_VALID_ACTIVE | EP4CFG_DIR_OUT | EP4CFG_TYPE_BULK ;
	delay() ;  EP6CFG = EP6CFG_VALID_ACTIVE | EP6CFG_DIR_IN  | EP6CFG_TYPE_BULK | EP6CFG_SIZE_512 | EP6CFG_BUF_DOUBLE ;
	delay() ;  EP8CFG = EP8CFG_VALID_ACTIVE | EP8CFG_DIR_IN  | EP8CFG_TYPE_BULK ;

	/**/
	delay() ;  EP2BCL = 0x80 ;
	delay() ;  EP2BCL = 0x80 ;
	delay() ;  EP2BCL = 0x80 ;
	delay() ;  EP4BCL = 0x80 ;
	delay() ;  EP4BCL = 0x80 ;

	EUSB      = 1 ;										/* USBINT 荞     */
	RESI      = 1 ;										/* W[荞  */
	EPIE     |= EPIE_EP1OUT ;							/* EP1OUT 荞ݗL */
	INTSETUP |= INTSETUP_AV2EN ;						/* I[gxN^L    */
														/* USB 荞ݐݒ    */
	USBIE     = USBIE_HSGRANT | USBIE_URES | USBIE_SUSP | USBIE_SUTOK | USBIE_SUDAV ;

	unl_cpu() ;

	disconnect( TRUE ) ;

	while( 1 ) {
		stall = 0 ;

		wai_sem( USB_SEMID ) ;

		switch( SETUPDAT[1] ) {
			case SC_GET_STATUS:
			{
				stall = getStatus() ;
				break ;
			}

			case SC_CLEAR_FEATURE:
			{
				stall = clearFeature() ;
				break ;
			}

			case SC_SET_FEATURE:
			{
				stall = setFeature() ;
				break ;
			}

//#			case SC_SET_ADDRESS:
			{
				break ;
			}

			case SC_GET_DESCRIPTOR:
			{
				stall = getDescriptor() ;
				break ;
			}

//#			case SC_SET_DESCRIPTOR:

			case SC_GET_CONFIGURATION:
			{
				EP0BUF[0] = 0 ;
				EP0BCH    = 0 ;
				EP0BCL    = 1 ;
				break ;
			}

			case SC_SET_CONFIGURATION:
			{
				break ;
			}

			case SC_GET_INTERFACE:
			{
				EP0BUF[0] = m_interface ;
				EP0BCH    = 0 ;
				EP0BCL    = 1 ;
				break ;
			}

			case SC_SET_INTERFACE:
			{
				m_interface = SETUPDAT[2] ;
				break ;
			}

//#			case SC_SYNC_FRAME:

			default:
			{
				stall = 1 ;
				break ;
			}
		}

		if( stall ) {
			EP0CS |= (EP0CS_HSNAK | EP0CS_STALL) ;
		} else {
			EP0CS |=  EP0CS_HSNAK ;
		}
	}
}


/* -------------------------------------------------------------------------------- */
/*   SUDAV 荞݃T[rX[`                                                 */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_sudav( void )
{
	isig_sem( USB_SEMID ) ;

	EXIF  &= ~EXIF_USBINT ;
	USBIRQ =  USBIRQ_SUDAV ;
}


/* -------------------------------------------------------------------------------- */
/*   SOF 荞݃T[rX[`                                                   */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_sof( void ) __interrupt(1) INTPROC
{
	EXIF  &= ~EXIF_USBINT ;
	USBIRQ =  USBIRQ_SOF ;
}


/* -------------------------------------------------------------------------------- */
/*   SUTOK 荞݃T[rX[`                                                 */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_sutok( void ) __interrupt(2) INTPROC
{
	EXIF  &= ~EXIF_USBINT ;
	USBIRQ =  USBIRQ_SUTOK ;
}


/* -------------------------------------------------------------------------------- */
/*   SUSPEND 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_suspend( void ) __interrupt(3) INTPROC
{
	EXIF  &= ~EXIF_USBINT ;
	USBIRQ =  USBIRQ_SUSP ;
}


/* -------------------------------------------------------------------------------- */
/*   USB RESET 荞݃T[rX[`                                             */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_usbreset( void ) __interrupt(4) INTPROC
{
	EXIF  &= ~EXIF_USBINT ;
	USBIRQ =  USBIRQ_URES ;
}


/* -------------------------------------------------------------------------------- */
/*   HISPEED 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_hispeed( void ) __interrupt(5) INTPROC
{
	EXIF  &= ~EXIF_USBINT ;
	USBIRQ =  USBIRQ_HSGRANT ;
}


/* -------------------------------------------------------------------------------- */
/*   EP0ACK 荞݃T[rX[`                                                */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep0ack( void ) __interrupt(6) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP0-IN 荞݃T[rX[`                                                */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep0in( void ) __interrupt(8) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP0-OUT 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep0out( void ) __interrupt(9) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP1-IN 荞݃T[rX[`                                                */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep1in( void ) __interrupt(10) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP1-OUT 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep1out( void ) __interrupt(11) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP2 荞݃T[rX[`                                                   */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep2( void ) __interrupt(12) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP4 荞݃T[rX[`                                                   */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep4( void ) __interrupt(13) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP6 荞݃T[rX[`                                                   */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep6( void ) __interrupt(14) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP8 荞݃T[rX[`                                                   */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep8( void ) __interrupt(15) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   In-Bulk-NAK 荞݃T[rX[`                                           */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_idn( void ) __interrupt(16) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP0PING 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep0ping( void ) __interrupt(18) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP1PING 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep1ping( void ) __interrupt(19) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP2PING 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep2ping( void ) __interrupt(20) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP4PING 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep4ping( void ) __interrupt(21) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP6PING 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep6ping( void ) __interrupt(22) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP8PING 荞݃T[rX[`                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep8ping( void ) __interrupt(23) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   ERRLIMIT 荞݃T[rX[`                                              */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_errlimit( void ) __interrupt(24) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP2ISOERR 荞݃T[rX[`                                             */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep2isoerr( void ) __interrupt(28) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP4ISOERR 荞݃T[rX[`                                             */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep4isoerr( void ) __interrupt(29) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP6ISOERR 荞݃T[rX[`                                             */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep6isoerr( void ) __interrupt(30) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   EP8ISOERR 荞݃T[rX[`                                             */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_isr_ep8isoerr( void ) __interrupt(31) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   USB eXgp^C}nh                                                 */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void usb_cychdr( void ) INTPROC
{
	static const uint16_t	padData[]={
								0x0100, 0x0500,		/*        */
								0x0100, 0x0500,		/*        */
								0x0900, 0x0500,		/*        */
								0x0900, 0x0500,		/*        */
								0x0400, 0x0500,		/*        */
								0x0600, 0x0500,		/* E       */
								0x0400, 0x0500,		/*        */
								0x0600, 0x0500,		/* E       */
								0x1500, 0x0500,		/* {^P */
								0x2500, 0x0500		/* {^Q */
							} ;
	static uint8_t			count=0 ;


	EP1INBUF[0] = (uint8_t )(padData[count] >> 8) ;
	EP1INBUF[1] = (uint8_t )(padData[count] >> 0) ;
	EP1INBC     = 2 ;

	if( ++count == (sizeof( padData )/sizeof( padData[0] )) ) {
		count = 0 ;
	}
}
