/* ================================================================================ */
/*   mw/rsfat.c                                                                     */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#include "rtos.h"
#include "mbr.h"
#include "fat.h"
#include "rsfat.h"

#define U16( m )					(((uint16_t )(m)[1]<<8)|(m)[0])
#define U32( m )					(((uint32_t )(m)[3]<<24)|((uint32_t )(m)[2]<<16)|((uint16_t )(m)[1]<<8)|(m)[0])


	enum {
		SECTOR_SIZE					=	512,						/* ZN^TCY                 */
		DIRENTRY_COUNT				=	512							/* [gfBNgGg */
	} ;

	/* Ɨ̈ */
	typedef union {
		MBR							mbr ;							/* MBR                  */
		PBR							pbr ;							/* PBR                  */
																	/* fBNgGg */
		DIRENTRY					dirEntry[SECTOR_SIZE/sizeof( DIRENTRY )] ;
		uint8_t						buff[SECTOR_SIZE] ;				/* ǂݍ݃obt@     */
	} WORK ;


static BOOL							m_mount ;						/* }EgtO                       */
static BOOL							m_openFile ;					/* t@CI[vtO               */
static BOOL							m_readEnd ;						/* ǂݍݏItO                   */
static __FAR   uint8_t				m_clusterSector ;				/* NX^̃ZN^             */
static __FAR   uint32_t				m_fatSector ;					/* FAT JnZN^                       */
static __FAR   uint32_t				m_rootSector ;					/* [gfBNgGgJnZN^ */
static __FAR   uint32_t				m_dataSector ;					/* f[^Jni-2 NX^jZN^      */
static __NEARI uint16_t				m_fileCluster ;					/* t@CǂݎNX^             */
static __NEARI uint8_t				m_fileSector ;					/* t@CǂݎNX^̃ZN^   */
static __NEARI uint32_t				m_fileSize ;					/* ct@CTCY                   */
static __NEARI uint16_t				m_readSize ;					/* ǂݍ݃TCY                       */
static RSFAT_WORK WORK				m_work ;						/* Ɨ̈                             */




/* ------------------------------------------------------------------------------------------------ */
/*   private                                                                                        */

/* -------------------------------------------------------------------------------- */
/*   f[^ǂݍ                                                               */
/*                                                                                  */
/*   IN : p_buff = ǂݍ݃obt@                                                 */
/*   OUT: E_OK                                                                      */
/*        RSFAT_E_FAIL                                                              */
/*        RSFAT_E_CORRUPTFILE                                                       */
/* -------------------------------------------------------------------------------- */
static int8_t readData( void __FAR *p_buff )
{
	int8_t	result ;

	/* ZN^擾 */
	if( m_fileSector == m_clusterSector ) {
		m_fileSector = 0 ;

		/* FAT ǂݍ */
		result = RSFAT_READ(m_fatSector+(m_fileCluster/(SECTOR_SIZE/2)), &m_work.buff[0], 1) ;
		if( result ) {
			return RSFAT_E_FAIL ;
		}
		
		m_fileCluster = (((uint16_t )m_work.buff[(m_fileCluster&255)*2+1] << 8) | m_work.buff[(m_fileCluster&255)*2+0]) ;

		/* 󂫁E\EsǃNX^EI[ȂُI */
		if( (uint16_t )(m_fileCluster-2) > 0xfff4 ) {
			return RSFAT_E_CORRUPTFILE ;
		}
	}

	/* f[^ǂݍ */
	result = RSFAT_READ(m_dataSector+((uint32_t )m_fileCluster*m_clusterSector)+m_fileSector, p_buff, 1) ;
	if( result ) {
		return RSFAT_E_FAIL ;
	}

	m_fileSector++ ;

	return E_OK ;
}




/* ------------------------------------------------------------------------------------------------ */
/*   public                                                                                         */

/* -------------------------------------------------------------------------------- */
/*   }Eg                                                                       */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: E_OK                                                                      */
/*        RSFAT_E_FAIL                                                              */
/*        RSFAT_E_MBR                                                               */
/*        RSFAT_E_PBR                                                               */
/* -------------------------------------------------------------------------------- */
int8_t rsfat_mount( void )
{
	int8_t		result ;
	uint16_t	value ;


	RSFAT_LOCK() ;

	/*  */
	m_mount = 0 ;

	do {
		result = RSFAT_INIT() ;
		if( result ) {
			result = RSFAT_E_FAIL ;
			break ;
		}

		/* MBR ǂݍ */
		result = RSFAT_READ(0, &m_work.buff[0], 1) ;
		if( result ) {
			result = RSFAT_E_FAIL ;
			break ;
		}

		/* VOl`ُȂI */
		if( U16( m_work.mbr.signature ) != 0xaa55 ) {
			result = RSFAT_E_MBR ;
			break ;
		}

		/* p[eBV^CvȂȂI */
		if( !m_work.mbr.partitionTable[0].type ) {
			result = RSFAT_E_MBR ;
			break ;
		}

		/* ANeButOُȂI */
/* SD J[h͗ĂĂȂ̂Ŗ */

		m_fatSector = U32( m_work.mbr.partitionTable[0].start ) ;

		/* p[eBVP PBR ǂݍ */
		result = RSFAT_READ(m_fatSector, &m_work.buff[0], 1) ;
		if( result ) {
			result = RSFAT_E_FAIL ;
			break ;
		}

		/* VOl`ُȂI */
		if( U16( m_work.pbr.BS_Signature ) != 0xaa55 ) {
			result = RSFAT_E_MBR ;
			break ;
		}

		/* PNX^̃ZN^ */
		m_clusterSector = m_work.pbr.BPB_SecPerClus ;
		if( !m_clusterSector ) {
			result = RSFAT_E_PBR ;
			break ;
		}

		/* \ZN^PłȂȂIiSD J[h͌ŒővHHHHj */
		if( U16( m_work.pbr.BPB_RsvdSecCnt ) != 1 ) {
			result = RSFAT_E_PBR ;
			break ;
		}

		/* FAT QłȂȂI */
		if( m_work.pbr.BPB_NumFATs != 2 ) {
			result = RSFAT_E_PBR ;
			break ;
		}

		/* [gfBNgGgႤȂI */
		if( U16( m_work.pbr.BPB_RootEntCnt ) != DIRENTRY_COUNT ) {
			result = RSFAT_E_PBR ;
			break ;
		}

		/* FAT ZN^ */
		value = U16( m_work.pbr.BPB_FATSz16 ) ;
		if( !value ) {
			result = RSFAT_E_PBR ;
			break ;
		}

		m_fatSector++ ;
		m_rootSector = m_fatSector  + (value * 2) ;
		m_dataSector = m_rootSector + (DIRENTRY_COUNT * sizeof( DIRENTRY ) / SECTOR_SIZE) - (2 * m_clusterSector) ;
		m_mount      = 1 ;

		/* I */
		result = E_OK ;
	} while( 0 ) ;

	RSFAT_UNLOCK() ;

	return result ;
}


/* -------------------------------------------------------------------------------- */
/*   t@CJ                                                                 */
/*                                                                                  */
/*   IN : p_fileName = t@C                                                   */
/*        p_fileSize = t@CTCYi[AhXiNULL ͖j                  */
/*   OUT: E_OK                                                                      */
/*        RSFAT_E_MOUNT                                                             */
/*        RSFAT_E_FAIL                                                              */
/*        RSFAT_E_FILENOTFOUND                                                      */
/*        RSFAT_E_ATTRIB                                                            */
/* -------------------------------------------------------------------------------- */
int8_t rsfat_open(const char __CODE *p_fileName, uint32_t __NEARI *p_fileSize)
{
	DIRENTRY	__FAR *p_dirEntry ;
	int8_t		result ;
	uint8_t		index, count, offset ;


	RSFAT_LOCK() ;

	/* }EgĂȂȂI */
	if( !m_mount ) {
		result = RSFAT_E_MOUNT ;
		goto rsfat_open_f00 ;
	}

	m_openFile = 0 ;
	m_readEnd  = 0 ;

	for(offset=0; offset<(DIRENTRY_COUNT*sizeof( DIRENTRY )/SECTOR_SIZE); offset++) {

		/* fBNgGgǂݍ */
		result = RSFAT_READ(m_rootSector+offset, &m_work.buff[0], 1) ;
		if( result ) {
			result = RSFAT_E_FAIL ;
			goto rsfat_open_f00 ;
		}

		p_dirEntry = &m_work.dirEntry[0] ;

		for(index=0; index<DIRENTRY_COUNT/sizeof( DIRENTRY ); index++,p_dirEntry++) {

			/* t@CȂȂI */
			if( !p_dirEntry->DIR_Name[0] ) {
				result = RSFAT_E_FILENOTFOUND ;
				goto rsfat_open_f00 ;
			}

			/* 폜t@CȂ疳 */
			if( p_dirEntry->DIR_Name[0] == 0xe5 ) {
				continue ;
			}

			/* t@CႤȂ疳 */
			for(count=0; count<sizeof( p_dirEntry->DIR_Name ); count++) {
				if( ((char )p_dirEntry->DIR_Name[count] !=p_fileName[count]) ) {
					break ;
				}
			}
			if( count != sizeof( p_dirEntry->DIR_Name ) ) {
				continue ;
			}

			/* fBNg{[xȂI */
			if( p_dirEntry->DIR_Attr & (ATTR_VOLUME_ID|ATTR_DIRECTORY) ) {
				result = RSFAT_E_ATTRIB ;
				goto rsfat_open_f00 ;
			}

			m_fileCluster = U16( p_dirEntry->DIR_FstClusLO ) ;
			m_fileSector  = 0 ;
			m_fileSize    = U32( p_dirEntry->DIR_FileSize ) ;
			m_readSize    = 0 ;

			/* 󂫁E\EsǃNX^EI[ȂُI */
			if( (uint16_t )(m_fileCluster-1) > 0xfff4 ) {
				result = RSFAT_E_CORRUPTFILE ;
				goto rsfat_open_f00 ;
			}

			if( p_fileSize != NULL ) {
				*p_fileSize = m_fileSize ;
			}

			m_openFile = 1 ;

			/* I */
			result = E_OK ;
			goto rsfat_open_f00 ;
		}
	}

	result = RSFAT_E_FILENOTFOUND ;

rsfat_open_f00:
	RSFAT_UNLOCK() ;

	return result ;
}


/* -------------------------------------------------------------------------------- */
/*   t@Cǂݍ                                                           */
/*                                                                                  */
/*   IN : p_buff = ǂݍ݃obt@                                                 */
/*        size   = ǂݍ݃TCY                                                   */
/*   OUT: E_OK                                                                      */
/*        RSFAT_E_FAIL                                                              */
/*        RSFAT_E_OPENFILE                                                          */
/*        RSFAT_E_CORRUPTFILE                                                       */
/*        RSFAT_E_CANTREAD                                                          */
/*        RSFAT_E_ENDOFFILE                                                         */
/* -------------------------------------------------------------------------------- */
int8_t rsfat_read(uint8_t __FAR *p_buff, uint16_t size)
{
	uint8_t		__FAR *p_srce ;
	int8_t		result, over ;
	uint16_t	copySize ;


	RSFAT_LOCK() ;

	/* t@CI[vłȂȂI */
	if( !m_openFile ) {
		result = RSFAT_E_OPENFILE ;
		goto rsfat_read_f00 ;
	}

	/* ȏt@Cǂݍ݂łȂȂI */
	if( m_readEnd ) {
		result = RSFAT_E_CANTREAD ;
		goto rsfat_read_f00 ;
	}

	/* t@CTCY */
	over = 0 ;
	if( !(m_fileSize&0xffff0000) ) {
		if( size > (uint16_t )m_fileSize ) {
			size = (uint16_t )m_fileSize ;
			over = 1 ;
		}
	}

	while( size ) {

		/* ǂݍݍς݃f[^݂ȂȂǂݍ */
		if( !m_readSize ) {

			/* ZN^TCY𒴂ĂȂ璼ڃf[^ǂݍ */
			if( size >= SECTOR_SIZE ) {
				result = readData( &p_buff[0] ) ;
				if( result ) {
					m_readEnd = 1 ;
					goto rsfat_read_f00 ;
				}
				p_buff     += SECTOR_SIZE ;
				size       -= SECTOR_SIZE ;
				m_fileSize -= SECTOR_SIZE ;
				continue ;
			}

			/* f[^ǂݍ */
			result = readData( &m_work.buff[0] ) ;
			if( result ) {
				m_readEnd = 1 ;
				goto rsfat_read_f00 ;
			}
			m_readSize = SECTOR_SIZE ;
		}

		p_srce = &m_work.buff[SECTOR_SIZE-m_readSize] ;

		/* Rs[TCY */
		if( size <= m_readSize ) {
			copySize    = size ;
			m_readSize -= size ;
			m_fileSize -= size ;
			size        = 0 ;
		} else {
			copySize    = m_readSize ;
			size       -= m_readSize ;
			m_fileSize -= m_readSize ;
			m_readSize  = 0 ;
		}

		/* Rs[ */
		do {
			*p_buff++ = *p_srce++ ;
		} while( --copySize ) ;
	}

	if( !m_fileSize ) {
		m_readEnd = 1 ;
	}

	result = E_OK ;

	if( over ) {
		result = RSFAT_E_ENDOFFILE ;
	}

rsfat_read_f00:
	RSFAT_UNLOCK() ;

	return result ;
}


/* -------------------------------------------------------------------------------- */
/*    t@C                                                              */
/*                                                                                  */
/*    IN : Ȃ                                                                     */
/*    OUT: Ȃ                                                                     */
/* -------------------------------------------------------------------------------- */
void rsfat_close( void )
{
	RSFAT_LOCK() ;

	m_openFile = 0 ;

	RSFAT_UNLOCK() ;
}
