/* ================================================================================ */
/*   test_app.c                                                                     */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#include "rtos.h"
#include "sc1602bs.h"
#include "at24c.h"


static BOOL							m_error ;						/* G[   */
static BOOL							m_led0 ;						/* LED0     */
static BOOL							m_led1 ;						/* LED1     */
static BOOL							m_led2 ;						/* LED2     */
static __FAR char					m_buff[32] ;					/* ƃobt@ */

static const char					m_hexAscii[]={
										'0', '1', '2', '3', '4', '5', '6', '7',
										'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
									} ;


/* -------------------------------------------------------------------------------- */
/*   PoCg̒l 16 iŕ\                                                   */
/*                                                                                  */
/*   IN : value = l                                                                */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
static void printHex8( uint8_t value )
{
	putchar( m_hexAscii[(value>> 4)&15] ) ;
	putchar( m_hexAscii[(value>> 0)&15] ) ;
}


/* -------------------------------------------------------------------------------- */
/*   QoCg̒l 16 iŕ\                                                   */
/*                                                                                  */
/*   IN : value = l                                                                */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
static void printHex16( uint16_t value )
{
	putchar( m_hexAscii[(value>>12)&15] ) ;
	putchar( m_hexAscii[(value>> 8)&15] ) ;
	putchar( m_hexAscii[(value>> 4)&15] ) ;
	putchar( m_hexAscii[(value>> 0)&15] ) ;
}


/* -------------------------------------------------------------------------------- */
/*   \                                                                   */
/*                                                                                  */
/*   IN : p_string =                                                          */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
static void puts( const char __CODE *p_string )
{
	while( *p_string != '\0' ) {
		if( *p_string == '\n' ) {
			putchar( '\r' ) ;
		}
		putchar( *p_string++ ) ;
	}
}


/* -------------------------------------------------------------------------------- */
/*   擾                                                                   */
/*                                                                                  */
/*   IN : p_string = i[AhX                                             */
/*        size     = i[TCY                                               */
/*   OUT: ̒                                                              */
/* -------------------------------------------------------------------------------- */
static uint8_t gets(char __FAR *p_string, uint8_t size)
{
	char		code ;
	uint8_t		count ;


	count = 0 ;
	size-- ;

	while( 1 ) {
		code = getchar() ;

		/* obNXy[X */
		if( code == '\b' ) {
			if( !count ) {
				continue ;
			}
			count-- ;
			puts( "\b\x1b[K" ) ;
			continue ;
		}

		/* s */
		if( code == '\r' ) {
			puts( "\n" ) ;
			p_string[count] = '\0' ;
			return count ;
		}

		if( count != size ) {
			p_string[count++] = code ;
			putchar( code ) ;
		}
	}
}


/* -------------------------------------------------------------------------------- */
/*   16 il֕ϊ                                                        */
/*                                                                                  */
/*   IN : p_string =                                                          */
/*   OUT: l                                                                        */
/* -------------------------------------------------------------------------------- */
static uint16_t hexStrToU16( const char __FAR *p_string )
{
	char		code ;
	uint8_t		count ;
	uint16_t	value ;

	/*  */
	m_error = 1 ;
	value   = 0 ;
	count   = 0 ;

	while( 1 ) {
		code = *p_string++ ;
		if( code == '\0' ) {

			/* ȂȂُI */
			if( !count ) {
				return 0 ;
			}
			break ;
		}

		/* I[o[ȂُI */
		if( ++count == 5 ) {
			return 0 ;
		}

		value <<= 4 ;
		if( (uint8_t )(code-'0') <= (uint8_t )('9'-'0') ) {
			value |= (code - '0') ;
		} else
		if( (uint8_t )(code-'A') <= (uint8_t )('F'-'A') ) {
			value |= (code - ('A' - 10)) ;
		} else
		if( (uint8_t )(code-'a') <= (uint8_t )('f'-'a') ) {
			value |= (code - ('a' - 10)) ;
		} else {
			return 0 ;
		}
	}

	m_error = 0 ;

	return value ;
}


/* -------------------------------------------------------------------------------- */
/*   16 oCgf[^_v                                                          */
/*                                                                                  */
/*   IN : p_data = f[^AhX                                                   */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
static void dumpData16( uint8_t __FAR *p_data )
{
	uint8_t		index ;


	for(index=0; index<16; index++) {
		printHex8( p_data[index] ) ;
	}

	puts( "  " ) ;

	for(index=0; index<16; index++) {
		if( (uint8_t )(p_data[index]-0x20) <= (uint8_t )(0x7e-0x20) ) {
			putchar( p_data[index] ) ;
		} else {
			putchar( '.' ) ;
		}
	}

	puts( "\n" ) ;
}


/* -------------------------------------------------------------------------------- */
/*   [_v                                                                 */
/*                                                                                  */
/*   IN : address = [AhX                                                */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
static void dumpMemory( uint16_t address )
{
	uint8_t		index ;


	for(index=0; index<16; index++) {
		printHex16( address ) ;

		putchar( ' ' ) ;

		dumpData16( (uint8_t __FAR *)address ) ;

		address += 16 ;
	}
}


/* -------------------------------------------------------------------------------- */
/*   AT24Cxxxx [_v                                                       */
/*                                                                                  */
/*   IN : address = [AhX                                                */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
#ifdef  CONFIG_AT24C
static void dumpAT24C( uint16_t address )
{
	uint8_t		count ;

	/*  */
	count = 16 ;

	if( at24c_read(AT24C_SLAVE_ADDRESS+1, address, &m_buff[0], 16) != E_OK ) {
		puts( "read error\n" ) ;
		return ;
	}

	while( 1 ) {
		printHex16( address ) ;

		putchar( ' ' ) ;

		dumpData16( &m_buff[0] ) ;

		if( !--count ) {
			break ;
		}

		if( at24c_cread(AT24C_SLAVE_ADDRESS+1, &m_buff[0], 16) != E_OK ) {
			puts( "read error\n" ) ;
			return ;
		}

		address += 16 ;
	}
}
#endif/*CONFIG_AT24C*/


/* -------------------------------------------------------------------------------- */
/*   C^XN                                                                   */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void main_tsk( void )
{
	/* LED i{ I/O ̏ rtos_init ōsj */
	APP_LED0( 0 ) ;
	APP_LED1( 0 ) ;
	APP_LED2( 0 ) ;
	APP_LED3( 0 ) ;

	puts( "\n" ) ;

	while( 1 ) {
		putchar( '>' ) ;

		gets(&m_buff[0], sizeof( m_buff )) ;

		switch( m_buff[0] ) {
			case 'd':	/* _v[ */
			{
				if( m_buff[1] != ' ' ) {
					continue ;
				}

				dumpMemory( hexStrToU16( &m_buff[2] ) & ~15 ) ;
				break ;
			}

			case 'l':	/* LED sE~ */
			{
				if( m_buff[1] != ' ' ) {
					break ;
				}

				if( m_buff[2] == 'r' ) {	/* s */
					sta_cyc( 1 ) ;
				} else
				if( m_buff[2] == 's' ) {	/* ~ */
					stp_cyc( 1 ) ;
				}
				break ;
			}

#ifdef  CONFIG_AT24C
			case 'i':
			{
				if( m_buff[1] != ' ' ) {
					break ;
				}

				dumpAT24C( hexStrToU16( &m_buff[2] ) & ~15 ) ;
				break ;
			}
#endif/*CONFIG_AT24C*/
		}
	}
}


/* -------------------------------------------------------------------------------- */
/*   Tu^XN                                                                     */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void sub_tsk( void )
{
	uint8_t		address ;


	address = ' ' ;

#ifdef  CONFIG_SC1602BS
	sc1602bs_init() ;
	sc1602bs_functionSet(1, 0) ;
	sc1602bs_entryModeSet(1, 0) ;
	sc1602bs_displayOnOffControl(1, 1, 1) ;
#endif/*CONFIG_SC1602BS*/

	while( 1 ) {
#ifdef  CONFIG_SC1602BS
		if( !(address&15) ) {
			sc1602bs_ddramAddressSet( (address&16) << 2 ) ;
		}
		address++ ;

		sc1602bs_cgramDdramDataWrite( address ) ;
#endif/*CONFIG_SC1602BS*/

		dly_tsk( 250 ) ;

		/* LED0 _ */
		m_led0 ^= 1 ;
		APP_LED0( m_led0 ) ;
	}
}


/* -------------------------------------------------------------------------------- */
/*   ACh^XN                                                                 */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void idle_tsk( void )
{
	while( 1 ) {
		/* ACh^XN API ̌Ăяo֎~iACh^XN~ƑɃ^XNȂȂ邽߁j */
	}
}


/* -------------------------------------------------------------------------------- */
/*   ^C}Pnh                                                           */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void cychdr1( void ) INTPROC
{
	/* LED1 _ */
	m_led1 ^= 1 ;
	APP_LED1( m_led1 ) ;
}


/* -------------------------------------------------------------------------------- */
/*   ^C}Qnh                                                           */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void cychdr2( void ) INTPROC
{
	/* LED2 _ */
	m_led2 ^= 1 ;
	APP_LED2( m_led2 ) ;
}
