/* ================================================================================ */
/*   test_lcd.c                                                                     */
/*                                                                                  */
/*   Copyright (C) 2014 lsluk, All rights reserved.                                 */
/*   http://www.geocities.jp/g_lsluk/index.html                                     */
/* ================================================================================ */
#include "rtos.h"
#include "aqm0802a.h"
#include "mcp3002.h"
#include "mpl115a1.h"
#include "lm024c9325.h"
#include "rsfat.h"

#define SPI_SEMID					5
#define RGB(r, g, b)				(((uint16_t )(r)<<11)|((uint16_t )(g)<<6)|b)

#undef putchar
#define putchar( c )				aqm0802a_writeDataToRam( c )

	/* rbg}bvt@Cwb_ */
	typedef struct {
		uint8_t						bfType[2] ;						/* t@C                         */
		uint8_t						bfSize[4] ;						/* t@CTCY                       */
		uint8_t						bfReserved1[2] ;				/* \P                               */
		uint8_t						bfReserved2[2] ;				/* \Q                               */
		uint8_t						bfOffBits[4] ;					/* t@C擪摜܂ł̃ItZbg */
	} BITMAPFILEHEADER ;

	/* rbg}bvwb_ */
	typedef struct {
		uint8_t						biSize[4] ;						/* TCY           */
		uint8_t						biWidth[4] ;					/*              */
		uint8_t						biHeight[4] ;					/* c             */
		uint8_t						biPlanes[2] ;					/* v[       */
		uint8_t						biBitCount[2] ;					/* rbg         */
		uint8_t						biCompression[4] ;				/* k`         */
		uint8_t						biSizeImage[4] ;				/* 摜f[^TCY */
		uint8_t						biXPelsPerMeter[4] ;			/* 𑜓x       */
		uint8_t						biYPelsPerMeter[4] ;			/* 𑜓x       */
		uint8_t						biClrUsed[4] ;					/* gppbg   */
		uint8_t						biClrImportant[4] ;				/* pbg       */
	} BITMAPINFOHEADER ;

	/* Ɨ̈ */
	typedef union {
																	/* ǂݍ݃obt@           */
		uint8_t						buff[LM024C9325_SCREEN_WIDTH*2] ;
		BITMAPFILEHEADER			bfh ;							/* rbg}bvt@Cwb_ */
	} WORK ;


static __FAR WORK					m_work ;						/* Ɨ̈   */
static uint16_t						m_adc[2] ;						/* A/D ϊl */


static const char					m_hexAscii[]={
										'0', '1', '2', '3', '4', '5', '6', '7',
										'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
									} ;

static const char					m_fileName[4][11+1]={
										"EZUSB_00BMP",
										"EZUSB_01BMP",
										"EZUSB_02BMP",
										"EZUSB_03BMP",
									} ;


/* -------------------------------------------------------------------------------- */
/*   PoCg̒l 16 iŕ\                                                   */
/*                                                                                  */
/*   IN : value = l                                                                */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
static void printHex8( uint8_t value )
{
	putchar( m_hexAscii[(value>> 4)&15] ) ;
	putchar( m_hexAscii[(value>> 0)&15] ) ;
}


/* -------------------------------------------------------------------------------- */
/*   QoCg̒l 16 iŕ\                                                   */
/*                                                                                  */
/*   IN : value = l                                                                */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
static void printHex16( uint16_t value )
{
	putchar( m_hexAscii[(value>>12)&15] ) ;
	putchar( m_hexAscii[(value>> 8)&15] ) ;
	putchar( m_hexAscii[(value>> 4)&15] ) ;
	putchar( m_hexAscii[(value>> 0)&15] ) ;
}


/* -------------------------------------------------------------------------------- */
/*   J[o[`                                                               */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
static void drawColorBar( void )
{
	static const uint16_t	colorData[]={
								RGB( 0, 0, 0), RGB( 0, 0,31), RGB(31, 0, 0), RGB(31, 0,31),
								RGB( 0,31, 0), RGB( 0,31,31), RGB(31,31, 0), RGB(31,31,31)
							} ;
	uint8_t					count ;
	uint16_t				index, color ;


	lm024c9325_setAddressH( 0 ) ;
	lm024c9325_setAddressV( 0 ) ;
	lm024c9325_beginWriteData() ;

	for(count=0; count<8; count++) {
		color = colorData[count] ;

		for(index=0; index<(320/8)*240; index++) {
			lm024c9325_writeData( color ) ;
		}
	}
}


/* -------------------------------------------------------------------------------- */
/*   rbg}bvt@C`                                                     */
/*                                                                                  */
/*   IN : p_fileName = t@C                                                   */
/*   OUT: E_OK                                                                      */
/*        E_SYS                                                                     */
/*        RSFAT_E_MOUNT                                                             */
/*        RSFAT_E_FAIL                                                              */
/*        RSFAT_E_FILENOTFOUND                                                      */
/*        RSFAT_E_ATTRIB                                                            */
/* -------------------------------------------------------------------------------- */
static int8_t drawBitmapFile( const char __CODE *p_fileName )
{
	int8_t		result ;
	uint8_t		xpos ;
	uint8_t		__FAR *p_data ;
	uint16_t	ypos, size ;

	/* t@C */
	wai_sem( SPI_SEMID ) ;

	do {
		/* SD J[hI */
		sdc_select() ;

		/* }Eg */
		result = rsfat_mount() ;
		if( result ) {
			break ;
		}

		/* t@CJ */
		result = rsfat_open(&p_fileName[0], NULL) ;
		if( result ) {
			break ;
		}

		/* rbg}bvt@Cwb_ǂݍ */
		result = rsfat_read((uint8_t *)&m_work.bfh, sizeof( m_work.bfh )) ;
		if( result ) {
			break ;
		}

		/* rbg}bvt@CłȂȂI */
		if( (m_work.bfh.bfType[0]!='B') || (m_work.bfh.bfType[1]!='M') ) {
			result = E_SYS ;
			break ;
		}

		/* t@C擪摜܂ł̃ItZbg傫ȂI */
		if( m_work.bfh.bfOffBits[2] | m_work.bfh.bfOffBits[3] ) {
			result = E_SYS ;
			break ;
		}

		/* wb_ȏȂI */
		size = ((uint16_t )m_work.bfh.bfOffBits[1] << 8) | m_work.bfh.bfOffBits[0] ;
		if( size < (sizeof( BITMAPFILEHEADER )+sizeof( BITMAPINFOHEADER )) ) {
			result = E_SYS ;
			break ;
		}
		size -= sizeof( BITMAPFILEHEADER ) ;

		/* 摜f[^̐擪擾 */
		while( size ) {
			if( size < sizeof( m_work.buff ) ) {
				result = rsfat_read(&m_work.buff[0], size) ;
				size   = 0 ;
			} else {
				result = rsfat_read(&m_work.buff[0], sizeof( m_work.buff )) ;
				size  -= sizeof( m_work.buff ) ;
			}
			if( result ) {
				break ;
			}
		}
	} while( 0 ) ;

	sig_sem( SPI_SEMID ) ;

	if( result != E_OK ) {
		return result ;
	}

	/* 摜f[^` */
	lm024c9325_setAddressH( 0 ) ;

	for(ypos=0; ypos<LM024C9325_SCREEN_HEIGHT; ypos++) {

		wai_sem( SPI_SEMID ) ;

		/* SD J[hI */
		sdc_select() ;

		/* Psǂݍ */
		result = rsfat_read(&m_work.buff[0], LM024C9325_SCREEN_WIDTH*2) ;

		sig_sem( SPI_SEMID ) ;

		if( result ) {
			return result ;
		}

		if( ypos != (LM024C9325_SCREEN_HEIGHT-1) ) {
			lm024c9325_vscroll( ypos + 1 ) ;
		} else {
			lm024c9325_vscroll( 0 ) ;
		}

		/* Ps` */
		lm024c9325_setAddressV( ypos ) ;
		lm024c9325_beginWriteData() ;

		p_data = &m_work.buff[0] ;
		for(xpos=0; xpos<LM024C9325_SCREEN_WIDTH; xpos++,p_data+=2) {
			lm024c9325_writeData( (((((uint16_t )p_data[1] << 8) | p_data[0]) & 0xffe0) << 1) | (p_data[0] & 0x1f) ) ;
		}
	}

	lm024c9325_setAddressV( 0 ) ;

	return E_OK ;
}


/* -------------------------------------------------------------------------------- */
/*   C^XN                                                                   */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void main_tsk( void )
{
	/*   */
	aqm0802a_init() ;
	aqm0802a_functionSet(1, 0, 0) ;
	aqm0802a_entryModeSet(1, 0) ;
	aqm0802a_displayOnOff(1, 1, 1) ;

	while( 1 ) {

		/* A/D ϊl擾 */
		wai_sem( SPI_SEMID ) ;

		mcp3002_select() ;
		m_adc[0] = mcp3002_read( MCP3002_MODE_SGL0 ) ;
		m_adc[1] = mcp3002_read( MCP3002_MODE_SGL1 ) ;

		sig_sem( SPI_SEMID ) ;

		aqm0802a_setDdramAddress( 0x00 ) ;
		printHex16( m_adc[0] ) ;

		aqm0802a_setDdramAddress( 0x40 ) ;
		printHex16( m_adc[1] ) ;

		dly_tsk( 200 ) ;
	}
}


/* -------------------------------------------------------------------------------- */
/*   Tu^XN                                                                     */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void sub_tsk( void )
{
	int8_t		result ;
	uint8_t		count ;

	/*  */
	count = 0 ;

	lm024c9325_init() ;

	/* J[o[` */
	drawColorBar() ;

	while( 1 ) {
		/* ǂݍ݊҂ */
		result = drawBitmapFile( &m_fileName[count][0] ) ;
		if( result ) {
			dly_tsk( 100 ) ;
			continue ;
		}

		if( ++count == (sizeof( m_fileName )/sizeof( m_fileName[0] )) ) {
			count = 0 ;
		}

		dly_tsk( 1000 ) ;
	}
}


/* -------------------------------------------------------------------------------- */
/*   ACh^XN                                                                 */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void idle_tsk( void )
{
	while( 1 ) {
		/* ACh^XN API ̌Ăяo֎~iACh^XN~ƑɃ^XNȂȂ邽߁j */
	}
}


/* -------------------------------------------------------------------------------- */
/*   ^C}Pnh                                                           */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void cychdr1( void ) INTPROC
{
}


/* -------------------------------------------------------------------------------- */
/*   ^C}Qnh                                                           */
/*                                                                                  */
/*   IN : Ȃ                                                                      */
/*   OUT: Ȃ                                                                      */
/* -------------------------------------------------------------------------------- */
void cychdr2( void ) INTPROC
{
}
